SystemOrganization addCategory: #'XML-Parser'!
SystemOrganization addCategory: #'XML-Parser-Nodes'!
SystemOrganization addCategory: #'XML-Parser-Namespaces'!
SystemOrganization addCategory: #'XML-Parser-DTD'!
SystemOrganization addCategory: #'XML-Parser-Exceptions'!

Trait named: #TXMLObservable
	uses: {}
	category: 'XML-Parser-Nodes'!

!TXMLObservable commentStamp: '<historical>' prior: 0!
This traits allows for observability to be grafted on to existing classes without altering their inheritance chain.

Classes using this trait must override #notifier to return an instance of XMLNotifier. Instances can then notify dependent objects of changes by sending themselves any of the messages under "notification," which each take a selector that observers are expected to understand and possibly one or more arguments to send with it.

To get an approximate list of notification selectors that a class using this trait expects its observers to implement, send the class #notifications.!

!TXMLObservable classSide methodsFor: 'utility' stamp: 'JAAyer 8/4/2010 21:06'!
notifications
	| notifications parser parsedMethods methodNode firstArgument |

	notifications := WriteStream on: (Array new: 10).
	parser := Parser new.
	parsedMethods := IdentityDictionary new.
	(self allMethodsInCategory: #notification)
		do: [:notifierSelector |
			(SystemNavigation default
				allSendersOf: notifierSelector
				inClass: self)
				do: [:methodReference |
					methodNode :=	
						parsedMethods
							at: methodReference selector
							ifAbsentPut:
								(parser
									parse: methodReference sourceCode
									class: self).
					methodNode nodesDo: [:node |
						(node isMessage
							and: [(node selector key asSymbol = notifierSelector)
								and: [(firstArgument := node arguments first) isLiteral]])
							ifTrue: [notifications nextPut: firstArgument key asSymbol]]]].
	^ notifications contents.! !

!TXMLObservable methodsFor: 'observing' stamp: 'JAAyer 6/15/2010 19:18'!
addObserver: anObserver
	self notifier addObserver: anObserver! !

!TXMLObservable methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 23:08'!
notifier
	"Override to return an instance of XMLNotifier"
	self explicitRequirement! !

!TXMLObservable methodsFor: 'notification' stamp: 'JAAyer 6/22/2010 05:32'!
notifyOf: aSelector with: anArgument
	self notifier notifyOf: aSelector with: anArgument! !

!TXMLObservable methodsFor: 'notification' stamp: 'JAAyer 6/22/2010 05:32'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument! !

!TXMLObservable methodsFor: 'notification' stamp: 'JAAyer 6/22/2010 05:32'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	self notifier
		notifyOf: aSelector
		with: aFirstArgument
		with: aSecondArgument
		with: aThirdArgument! !

!TXMLObservable methodsFor: 'notification' stamp: 'JAAyer 7/21/2010 09:14'!
notifyOf: aSelector withArguments: anArray
	self notifier notifyOf: aSelector withArguments: anArray! !

!TXMLObservable methodsFor: 'observing' stamp: 'JAAyer 6/15/2010 19:18'!
removeObserver: anObserver
	self notifier removeObserver: anObserver! !

!ByteString methodsFor: '*xml-parser' stamp: 'JAAyer 8/11/2010 22:36'!
applyLanguageInfomation: aLanguageEnvironment! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 8/11/2010 22:36'!
applyLanguageInfomation: aLanguageEnvironment
	| leadingChar |

	leadingChar := aLanguageEnvironment leadingChar.
	self withIndexDo: [:each :i |
		each asciiValue > 255
			ifTrue: [
				self
					at: i
					put: (Character leadingChar: leadingChar code: each asUnicode)]]! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 4/14/2010 21:26'!
asXMLString
	^ XMLString string: self! !

!String methodsFor: '*xml-parser' stamp: 'JAAyer 8/9/2010 14:46'!
splitQualifiedNameInto: aTwoArgumentBlock
	"Split the name into namespace prefix and local name (the block arguments)."
	| i |

	i := self size.
	[i > 1]
		whileTrue: [
			((self at: i) == $:)
				ifTrue: [
					^ aTwoArgumentBlock
						value: (self copyFrom: 1 to: i - 1)
						value: (self copyFrom: i + 1 to: self size)].
			i := i - 1].
	^ aTwoArgumentBlock
		value: ''
		value: self! !

!WriteStream methodsFor: '*xml-parser' stamp: 'JAAyer 6/17/2010 19:53'!
stringContents
	| newSize |

	newSize := position.
	^ (collection class new: newSize)	
		replaceFrom: 1
		to: newSize
		with: collection
		startingAt: 1! !

Error subclass: #XMLParserException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLDOMException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLNamespaceException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLParsingException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

XMLParserException subclass: #XMLWellFormednessException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

Warning subclass: #XMLParserWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Exceptions'!

SequenceableCollection subclass: #XMLOrderedList
	instanceVariableNames: 'collection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLOrderedList commentStamp: '<historical>' prior: 0!
This class functions as an OrderedCollection that will not raise exceptions when attempts are made to access absent objects. This means messages like #at:, #after:, and #first will return nil when the requested object cannot be retrieved. Messages that modify the collection based on the presence or absence of other objects such as #add:after: or #at:put: will still raise exceptions as usual.

!

XMLOrderedList subclass: #XMLObservableList
	uses: TXMLObservable
	instanceVariableNames: 'notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLObservableList commentStamp: '<historical>' prior: 0!
This class is an is an ordered list that sends notifications to observers when objects are added to it or removed from it. Subclasses should override #adding:, #added:, and #removed: to send different notifications.!

XMLObservableList class
	uses: TXMLObservable classTrait
	instanceVariableNames: ''!

XMLObservableList subclass: #XMLAttributeList
	instanceVariableNames: 'attributesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLAttributeList commentStamp: 'JAAyer 7/10/2010 05:45' prior: 0!
This class represents an ordered list of attribute nodes that also keeps a dictionary mapping attribute names to nodes for faster lookups. You are unlikely to need to use this class directly.!

!XMLAttributeList methodsFor: 'adding' stamp: 'JAAyer 10/15/2010 12:41'!
addName: aName value: aValue
	^ self addLast:
		(XMLAttribute
			name: aName
			value: aValue)! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 11/15/2010 05:00'!
added: anObject
	anObject addObserver: self.
	self attributesByName
		at: anObject name ifPresent: [self removeAttributeNamed: anObject name];
		at: anObject name put: anObject.

	self notifyOf: #addedAttribute: with: anObject.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 10/24/2010 18:12'!
adding: anObject
	anObject element
		ifNotNil: [XMLDOMException signal: 'Cannot add attribute node from other element']! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 11/20/2010 08:03'!
asDictionary
	^ OrderPreservingDictionary newFrom: self asNameValueAssociations! !

!XMLAttributeList methodsFor: 'converting' stamp: 'JAAyer 6/25/2010 22:15'!
asNameValueAssociations
	^ self collect: [:each | each name -> each value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 11:02'!
attributeNamed: aName
	^ self attributeNamed: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 11:02'!
attributeNamed: aName ifAbsent: aBlock
	^ self attributesByName at: aName ifAbsent: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 13:05'!
attributeNamed: aLocalName namespaceURI: aUri
	^ self attributeNamed: aLocalName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 13:06'!
attributeNamed: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self
		detect: [:each | (each isNamed: aLocalName) and: [each namespaceURI = aUri]]
		ifNone: aBlock! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self collect: [:each | each name]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self do: [:each | aTwoArgumentBlock value: each name value: each value]! !

!XMLAttributeList methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self do: [:each | aBlock value: each name]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName
	^ self attributeValueAt: aName ifAbsent: [self defaultValue]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:11'!
attributeValueAt: aName ifAbsent: aBlock
	^ (self attributesByName
		at: aName
		ifAbsent: [^ aBlock value]) value! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 7/11/2010 03:34'!
attributeValueAt: aName ifAbsentPut: aBlock
	^ self
		attributeValueAt: aName
		ifAbsent: [self attributeValueAt: aName put: aBlock value]! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:12'!
attributeValueAt: aName put: aValue
	(self includesAttributeNamed: aName)
		ifTrue: [(self attributeNamed: aName) value: aValue]
		ifFalse: [self addName: aName value: aValue].
	^ aValue.! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 6/10/2010 05:51'!
attributesByName
	^ attributesByName ifNil: [attributesByName := Dictionary new]! !

!XMLAttributeList methodsFor: 'testing' stamp: 'JAAyer 6/10/2010 11:41'!
includesAttributeNamed: aName
	^ self attributesByName includesKey: aName! !

!XMLAttributeList methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 11:05'!
removeAttributeNamed: aName
	^ self remove: (self attributeNamed: aName).! !

!XMLAttributeList methodsFor: 'private' stamp: 'JAAyer 10/24/2010 18:27'!
removed: anAttribute
	anAttribute removeObserver: self.
	self attributesByName removeKey: anAttribute name.
	self notifyOf: #removedAttribute: with: anAttribute.! !

!XMLAttributeList methodsFor: 'notifications' stamp: 'JAAyer 6/15/2010 15:13'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self attributesByName
		removeKey: anOldName;
		at: aNewName put: anAttribute! !

XMLObservableList subclass: #XMLInternalSubsetList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLInternalSubsetList commentStamp: 'JAAyer 7/27/2010 12:45' prior: 0!
This class represents the list of nodes that comprise a document's internal DTD subset.!

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 8/25/2010 00:01'!
added: aNode
	self notifyOf: #addedNode: with: aNode.! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 7/22/2010 11:09'!
adding: aNode! !

!XMLInternalSubsetList methodsFor: 'private' stamp: 'JAAyer 8/25/2010 00:01'!
removed: aNode
	self notifyOf: #removedNode: with: aNode.! !

XMLObservableList subclass: #XMLNodeList
	instanceVariableNames: 'elements elementsByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeList commentStamp: '<historical>' prior: 0!
This class represents an ordered list of child nodes for some kind of node with elements, probably either an XML document or element object. It treats element nodes specially, keeping both an ordered list of child elements and a dictionary that allows for faster retrieval of element(s) by qualified name or local name only.!

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 10/24/2010 18:18'!
added: aNode
	aNode addObserver: self.
	aNode isElement
		ifTrue: [
			self last == aNode
				ifTrue: [self elementAddedAtEnd: aNode]
				ifFalse: [self elementInsertedBeforeEnd: aNode].
			self notifyOf: #addedElement: with: aNode].
	self notifyOf: #addedNode: with: aNode.! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 7/22/2010 11:04'!
adding: aNode
	aNode hasParent
		ifTrue: [XMLDOMException signal: 'Cannot add child node that already has a parent']! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 10/19/2010 10:57'!
elementAddedAtEnd: anElement
	self elements add: anElement.
	self
		elementsByNameAt: anElement name
		addElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				addElement: anElement]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/5/2010 23:03'!
elementInsertedBeforeEnd: anElement
	| elementBefore |

	(elementBefore := self findElementBefore: anElement)
		ifNil: [self elements addFirst: anElement]
		ifNotNil: [self elements add: anElement after: elementBefore].

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 11:43'!
elementNamed: aName
	^ self elementNamed: aName ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 11:41'!
elementNamed: aName ifAbsent: aBlock
	| elementsWithName |

	elementsWithName := self elementsByName at: aName ifAbsent: [^ aBlock value].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName first]
		ifFalse: [elementsWithName]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 08:56'!
elementNamed: aName namespaceURI: aUri
	^ self elementNamed: aName namespaceURI: aUri ifAbsent: [self defaultValue]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 08:56'!
elementNamed: aName namespaceURI: aUri ifAbsent: aBlock
	^ (self elementsNamed: aName)
		detect: [:each | each namespaceURI = aUri]
		ifNone: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 5/19/2010 18:25'!
elements
	^ elements ifNil: [elements := self species new]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 6/10/2010 11:07'!
elementsByName
	^ elementsByName ifNil: [elementsByName := Dictionary new].! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 7/28/2010 08:50'!
elementsByNameAt: aName addElement: anElement
	| elementsWithName |

	elementsWithName := 
		self elementsByName
			at: aName
			ifAbsent: [
				self elementsByName at: aName put: anElement.
				^ self].

	elementsWithName isCollection
		ifTrue: [elementsWithName add: anElement]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName with: anElement)]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/5/2010 23:13'!
elementsByNameAt: aName addElementFirst: anElement
	| elementsWithName |

	elementsWithName :=
		self elementsByName
			at: aName
			ifAbsent: [
				self elementsByName at: aName put: anElement.
				^ self].

	elementsWithName isCollection
		ifTrue: [elementsWithName addFirst: anElement]
		ifFalse: [
			self elementsByName
				at: aName
				put: [self species with: anElement with: elementsWithName]].! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 11/5/2010 23:10'!
elementsByNameAt: aName insertElement: anElement
	| elementsWithName elementBefore |

	(elementBefore := self findElementBefore: anElement withName: aName)
		ifNil: [^ self elementsByNameAt: aName addElementFirst: anElement].

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [elementsWithName add: anElement after: elementBefore]
			ifFalse: [
				self elementsByName
					at: aName
					put: (self species with: elementBefore with: anElement)]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 7/21/2010 12:07'!
elementsByNameAt: aName removeElement: anElement
	| elementsWithName |

	(elementsWithName := self elementsByName at: aName)
		isCollection
			ifTrue: [
				elementsWithName remove: anElement.
				elementsWithName ifEmpty: [self elementsByName removeKey: aName]]
			ifFalse: [self elementsByName removeKey: aName]! !

!XMLNodeList methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self elements do: aBlock! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:53'!
elementsNamed: aName
	| elementsWithName |

	elementsWithName :=
		self elementsByName
			at: aName
			ifAbsent: [^ self species new].

	^ elementsWithName isCollection
		ifTrue: [elementsWithName]
		ifFalse: [
			self elementsByName
				at: aName
				put: (self species with: elementsWithName)]! !

!XMLNodeList methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:54'!
elementsNamed: aName namespaceURI: aUri
	^ (self elementsNamed: aName) select: [:each | each namespaceURI = aUri]! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 6/9/2010 08:21'!
findElementBefore: anElement
	| elementBefore |

	elementBefore := nil.
	self do: [:each |
		each isElement
			ifTrue: [
				(each = anElement)
					ifTrue: [^ elementBefore].
				elementBefore := each]].
	^ elementBefore.! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 6/9/2010 08:22'!
findElementBefore: anElement withName: aName
	| lastNamedElement |

	lastNamedElement := nil.
	self do: [:each |
		(each isElement and: [each isNamed: aName])
			ifTrue: [
				(each = anElement)
					ifTrue: [^ lastNamedElement].
				lastNamedElement := each]].
	^ lastNamedElement.! !

!XMLNodeList methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 12:20'!
hasElements
	^ elements notNil and: [elements notEmpty]! !

!XMLNodeList methodsFor: 'testing' stamp: 'JAAyer 6/10/2010 11:43'!
includesElementNamed: aName
	^ self elementsByName includesKey: aName! !

!XMLNodeList methodsFor: 'private' stamp: 'JAAyer 10/24/2010 18:26'!
removed: aNode
	aNode removeObserver: self.
	aNode isElement
		ifTrue: [
			self elements remove: aNode.
			self elementsByNameAt: aNode name removeElement: aNode.
			aNode hasPrefix
				ifTrue: [self elementsByNameAt: aNode localName removeElement: aNode].
			self notifyOf: #removedElement: with: aNode].
	self notifyOf: #removedNode: with: aNode.! !

!XMLNodeList methodsFor: 'notifications' stamp: 'JAAyer 10/19/2010 10:55'!
renamedElement: anElement from: anOldName to: aNewName
	self elementsByNameAt: anOldName removeElement: anElement.
	anOldName splitQualifiedNameInto: [:oldPrefix :oldLocalName |
		oldPrefix ifNotEmpty: [
			self
				elementsByNameAt: oldLocalName
				removeElement: anElement]].

	self elementsByNameAt: anElement name insertElement: anElement.
	anElement hasPrefix
		ifTrue: [
			self
				elementsByNameAt: anElement localName
				insertElement: anElement].! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject after: aPriorObject
	^ self with: [super add: anObject after: aPriorObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:56'!
add: anObject afterIndex: anIndex
	^ self with: [super add: anObject afterIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 14:46'!
add: anObject before: aSubsequentObject
	^ self with: [super add: anObject before: aSubsequentObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
add: anObject beforeIndex: anIndex
	^ self with: [super add: anObject beforeIndex: anIndex] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addFirst: anObject
	^ self with: [super addFirst: anObject] add: anObject! !

!XMLObservableList methodsFor: 'adding' stamp: 'JAAyer 7/22/2010 10:57'!
addLast: anObject
	^ self with: [super addLast: anObject] add: anObject! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 6/22/2010 05:29'!
added: anObject
	self notifyOf: #added:to: with: anObject with: self.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 7/22/2010 11:01'!
adding: anObject
	self notifyOf: #adding:to: with: anObject with: self! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 11:06'!
at: anIndex put: anObject
	| presentObject |

	(presentObject := self at: anIndex)
		ifNotNil: [
			self
				with: [super at: anIndex put: nil]
				remove: presentObject].

	^ self with: [super at: anIndex put: anObject] add: anObject.! !

!XMLObservableList methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 10:50'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 7/22/2010 10:58'!
remove: anObject ifAbsent: aBlock
	^ self with: [super remove: anObject ifAbsent: [^ aBlock value]] remove: anObject! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeAt: anIndex
	^ self with: [super removeAt: anIndex] remove: (self at: anIndex)! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeFirst
	^ self removeAt: 1! !

!XMLObservableList methodsFor: 'removing' stamp: 'JAAyer 6/11/2010 09:03'!
removeLast
	^ self removeAt: self size! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 6/22/2010 05:29'!
removed: anObject
	self notifyOf: #removed:from: with: anObject with: self.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 7/22/2010 11:00'!
with: aBlock add: anObject
	self
		adding: anObject;
		with: [aBlock value] onError: [^ nil];
		added: anObject.
	^ anObject.! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 6/11/2010 09:03'!
with: aBlock onError: errorBlock
	^ aBlock
		on: Error
		do: [:error |
			errorBlock valueWithPossibleArgument: error.
			error signal]! !

!XMLObservableList methodsFor: 'private' stamp: 'JAAyer 7/22/2010 11:05'!
with: aBlock remove: anObject
	self
		with: [aBlock value] onError: [^ nil];
		removed: anObject.
	^ anObject.! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:44'!
new
	^ self new: 10! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 16:33'!
new: aSize
	^ self basicNew setCollection: (OrderedCollection new: aSize)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:41'!
new: aSize withAll: anObject
	^ self basicNew setCollection: (OrderedCollection new: aSize withAll: anObject)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:42'!
newFrom: aCollection
	^ self basicNew setCollection: (OrderedCollection newFrom: aCollection)! !

!XMLOrderedList class methodsFor: 'instance creation' stamp: 'JAAyer 8/4/2010 14:43'!
ofSize: aSize
	^ self new: aSize withAll: nil! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:01'!
add: anObject
	^ self addLast: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:08'!
add: anObject after: aPriorObject
	^ collection add: anObject after: aPriorObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject afterIndex: anIndex
	^ collection add: anObject afterIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:09'!
add: anObject before: aSubsequentObject
	^ collection add: anObject before: aSubsequentObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:10'!
add: anObject beforeIndex: anIndex
	^ collection add: anObject beforeIndex: anIndex! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:02'!
addAll: aCollection
	^ self addAllLast: aCollection! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:10'!
addAllFirst: aCollection
	aCollection reverseDo: [:each | self addFirst: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:11'!
addAllLast: aCollection
	aCollection do: [:each | self addLast: each].
	^ aCollection.! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:11'!
addFirst: anObject
	^ collection addFirst: anObject! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 13:12'!
addLast: anObject
	^ collection addLast: anObject! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:48'!
after: anObject
	^ collection after: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'converting' stamp: 'JAAyer 8/4/2010 12:53'!
asArray
	^ collection asArray! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
at: anIndex
	^ collection at: anIndex ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'adding' stamp: 'JAAyer 8/4/2010 15:12'!
at: anIndex ifAbsentPut: aValue
	^ self
		at: anIndex
		ifAbsent: [self at: anIndex put: aValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:25'!
at: anIndex put: aValue
	^ collection at: anIndex put: aValue! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 00:55'!
atLast: anIndexFromEnd
	^ self atLast: anIndexFromEnd ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:50'!
before: anObject
	^ collection before: anObject ifAbsent: [self defaultValue]! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:14'!
capacity
	^ collection capacity! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
collect: aBlock
	^ self copyThenSetCollection: (collection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
collect: aBlock from: aStartIndex to: anEndIndex
	^ self copyThenSetCollection:
		(collection collect: aBlock from: aStartIndex to: anEndIndex)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
collect: aCollectBlock thenSelect: aSelectBlock
	^ self copyThenSetCollection:
		(collection collect: aCollectBlock thenSelect: aSelectBlock)! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 8/4/2010 14:54'!
copy
	^ self species basicNew setCollection: collection copy! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 3/12/2010 07:30'!
copyEmpty
	^ self species new: self size! !

!XMLOrderedList methodsFor: 'copying' stamp: 'JAAyer 10/18/2010 11:21'!
copyFrom: aStart to: anEnd
	^ self copyThenSetCollection:
		(collection copyFrom: (aStart max: 1) to: (self size min: anEnd))! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 10/18/2010 11:21'!
copyThenSetCollection: aCollection
	^ self class basicNew setCollection: aCollection! !

!XMLOrderedList methodsFor: 'defaults' stamp: 'JAAyer 3/12/2010 06:48'!
defaultValue
	^ nil! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 12:49'!
do: aBlock
	collection do: aBlock! !

!XMLOrderedList methodsFor: 'inspecting' stamp: 'JAAyer 8/4/2010 13:21'!
inspectorClass
	^ OrderedCollectionInspector! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:33'!
remove: anObject ifAbsent: aBlock
	^ collection remove: anObject ifAbsent: aBlock! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 16:00'!
removeAll
	[self isEmpty] whileFalse: [self removeLast]! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 15:46'!
removeAt: anIndex
	^ collection removeAt: anIndex! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeFirst
	^ collection removeFirst! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeFirst: aNumber
	^ collection removeFirst: aNumber! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeLast
	^ collection removeLast! !

!XMLOrderedList methodsFor: 'removing' stamp: 'JAAyer 8/4/2010 13:34'!
removeLast: aNumber
	^ collection removeLast: aNumber! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 13:18'!
replaceFrom: aStartingIndex to: anEndingIndex with: aCollection startingAt: aReplacementStart
	collection
		replaceFrom: aStartingIndex
		to: anEndingIndex
		with: aCollection
		startingAt: aReplacementStart! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:35'!
reverseDo: aBlock
	collection reverseDo: aBlock! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
select: aBlock
	^ self copyThenSetCollection: (collection select: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
select: aSelectBlock thenCollect: aCollectBlock
	^ self copyThenSetCollection:
		(collection select: aSelectBlock thenCollect: aCollectBlock)! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 8/4/2010 13:38'!
setCollection: aCollection
	collection := aCollection! !

!XMLOrderedList methodsFor: 'accessing' stamp: 'JAAyer 8/4/2010 12:49'!
size
	^ collection size! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:43'!
sort
	collection sort! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 8/4/2010 13:43'!
sort: aSortBlock
	collection sort: aSortBlock! !

!XMLOrderedList methodsFor: 'private' stamp: 'JAAyer 3/12/2010 11:02'!
species
	^ XMLOrderedList! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
with: aCollection collect: aBlock
	^ self copyThenSetCollection: (collection with: aCollection collect: aBlock)! !

!XMLOrderedList methodsFor: 'enumerating' stamp: 'JAAyer 10/18/2010 11:21'!
withIndexCollect: aTwoArgumentBlock
	^ self copyThenSetCollection: (collection withIndexCollect: aTwoArgumentBlock)! !

Object subclass: #DTDEntityDeclaration
	instanceVariableNames: 'name value ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

DTDEntityDeclaration class
	instanceVariableNames: 'contextBehavior'!

DTDEntityDeclaration class
	instanceVariableNames: 'contextBehavior'!

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:14'!
behaviorForContext: aContext
	^self contextBehavior at: aContext! !

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:15'!
contextBehavior
	^contextBehavior! !

!DTDEntityDeclaration class methodsFor: 'class initialization' stamp: 'mir 1/15/2002 18:02'!
initialize
	"DTDEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #include ;
		at: #attributeValueContent put: #includedInLiteral ;
		at: #attributeValue put: #forbidden ;
		at: #entityValue put: #bypass ;
		at: #dtd put: #forbidden ! !

!DTDEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:27'!
leadIn
	^'&'! !

!DTDEntityDeclaration class methodsFor: 'instance creation' stamp: 'mir 11/16/2000 20:13'!
name: aString value: aValueString
	^self new
		name: aString;
		value: aValueString! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:29'!
bypass
	"Return my reference as is."
	^self reference! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:29'!
forbidden
	self error: 'Forbidden reference usage'! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'JAAyer 11/20/2010 14:24'!
include
	"Return my expanded value."
	^ value ifNil: [XMLParserWarning signal: 'XML undefined entity ' , name printString]! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 18:06'!
includedInLiteral
	"Return my expanded value."
	^self include! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 1/4/2002 19:40'!
name
	^name! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'JAAyer 3/8/2010 04:52'!
name: aString
	name := aString! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:22'!
ndata
	^ndata! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 12/8/2000 17:22'!
ndata: aString
	ndata := aString! !

!DTDEntityDeclaration methodsFor: 'behaviors' stamp: 'JAAyer 3/11/2010 09:50'!
reference
	"Return my reference as is."
	^self class leadIn, self name, ';'! !

!DTDEntityDeclaration methodsFor: 'invocation' stamp: 'mir 11/16/2000 21:23'!
registerIn: aParser
	aParser entity: self name put: self! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 11/16/2000 10:54'!
value
	^value! !

!DTDEntityDeclaration methodsFor: 'accessing' stamp: 'mir 11/16/2000 10:55'!
value: aString
	value := aString! !

!DTDEntityDeclaration methodsFor: 'invocation' stamp: 'mir 1/15/2002 15:08'!
valueForContext: aContext
	^self perform: (self class behaviorForContext: aContext)! !

DTDEntityDeclaration subclass: #DTDExternalEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDExternalEntityDeclaration classSide methodsFor: 'class initialization' stamp: 'mir 1/14/2002 18:15'!
initialize
	"DTDExternalEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #include ;
		at: #attributeValueContent put: #includedInLiteral ;
		at: #attributeValue put: #forbidden ;
		at: #entityValue put: #bypass ;
		at: #dtd put: #forbidden ! !

DTDEntityDeclaration subclass: #DTDParameterEntityDeclaration
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-DTD'!

!DTDParameterEntityDeclaration class methodsFor: 'class initialization' stamp: 'mir 1/14/2002 18:15'!
initialize
	"DTDParameterEntityDeclaration initialize"

	contextBehavior := Dictionary new.
	contextBehavior
		at: #content put: #notRecognized: ;
		at: #attributeValueContent put: #notRecognized: ;
		at: #attributeValue put: #notRecognized: ;
		at: #entityValue put: #include: ;
		at: #dtd put: #includePE:! !

!DTDParameterEntityDeclaration class methodsFor: 'accessing' stamp: 'mir 11/16/2000 20:27'!
leadIn
	^'%'! !

!DTDParameterEntityDeclaration methodsFor: 'behaviors' stamp: 'mir 1/15/2002 11:30'!
includePE
	"Return my expanded value."
	^self include! !

!DTDParameterEntityDeclaration methodsFor: 'behaviors' stamp: 'JAAyer 10/15/2010 15:02'!
notRecognized
	XMLWellFormednessException signal: 'Malformed entity.'! !

!DTDParameterEntityDeclaration methodsFor: 'invocation' stamp: 'mir 11/28/2000 17:26'!
registerIn: aParser
	aParser parameterEntity: self name put: self! !

Object subclass: #SAXHandler
	instanceVariableNames: 'driver eod'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXHandler commentStamp: 'JAAyer 3/9/2010 12:11' prior: 0!
A SAXHandler is an XML parser that uses event handling. The acronym "SAX" refers to Java's "Simple API for XML," on which SAXHandler's API is based. To use this class, create a subclass and override handlers in "content" and "lexical" as needed. To instantiate it you can send #on: or any of the #parse- messages. The former creates a new parser on the given input stream and returns it; the instance can then be sent #parseDocument to parse the input. The #parse- messages take some input source and parse it, returning the result of #parseDocument.

By default, namespace support is enabled.!

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:40'!
on: aStringOrStream
	^ self new driver: (SAXDriver on: aStringOrStream)! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:40'!
onFileNamed: aFileName
	^ self on: (FileStream readOnlyFileNamed: aFileName)! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:21'!
parse: aStringOrStream
	^ (self on: aStringOrStream) parseDocument! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:30'!
parse: aStringOrStream usingNamespaces: aBoolean
	^ (self on: aStringOrStream)
		usesNamespaces: aBoolean;
		parseDocument! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 02:03'!
parseDTDFrom: aStream
	| parser |

	parser := self on: aStream.
	parser driver startParsingMarkup.
	^ parser parseDocument.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:32'!
parseDocumentFrom: aStringOrStream
	^ self parse: aStringOrStream.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:32'!
parseDocumentFrom: aStringOrStream useNamespaces: aBoolean
	^ self parse: aStringOrStream usingNamespaces: aBoolean.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:42'!
parseDocumentFromFileNamed: aFileName
	self deprecated: 'use #parseFileNamed: instead'.

	^ self parseFileNamed: aFileName.! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:43'!
parseFileNamed: aFileName
	^ (self onFileNamed: aFileName) parseDocument! !

!SAXHandler class methodsFor: 'instance creation' stamp: 'JAAyer 11/13/2010 20:41'!
parserOnFileNamed: aFileName
	self deprecated: 'use #onFileNamed: instead'.

	^ self onFileNamed: aFileName.! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:27'!
characters: aString
	"This call corresponds to the Java SAX call
	characters(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
comment: aCommentString
	"This call corresponds to the Java SAX ext call
	comment(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 22:56'!
currentScope
	^ self driver currentScope! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 3/23/2010 15:55'!
driver
	^ driver! !

!SAXHandler methodsFor: 'accessing' stamp: 'mir 12/7/2000 15:34'!
driver: aDriver
	driver := aDriver.
	driver saxHandler: self! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 2/21/2010 15:13'!
endCData
	"This call corresponds to the Java SAX ext call
	endCData()."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 1/8/2002 18:26'!
endDocument
	"This call corresponds to the Java SAX call
	endDocument()."
	eod := true! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:23'!
endElement: aQualifiedName
! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:50'!
endElement: aQualifiedName prefix: aPrefix uri: aUri
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self endElement: aQualifiedName! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 17:06'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	"This call corresponds to the Java SAX call
	endElement(java.lang.String namespaceURI, java.lang.String localName, java.lang.String qName).
	By default this call is mapped to the following more convenient call:"

	self
		invokeDeprecated: #endElement:namespace:namespaceURI:qualifiedName:
		withArguments: (Array with: aLocalName with: aPrefix with: aUri with: aQualifiedName)
		orForwardTo: #endElement:prefix:uri:
		withArguments: (Array with: aQualifiedName with: aPrefix with: aUri)! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:23'!
endEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	endEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
endPrefixMapping: prefix
	"This call corresonds to the Java SAX call
	endPrefixMapping(java.lang.String prefix)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 3/16/2010 09:27'!
eod
	^ eod ifNil: [eod := false]! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:25'!
ignorableWhitespace: aString
	"This call corresonds to the Java SAX call
	ignorableWhitespace(char[] ch, int start, int length)."! !

!SAXHandler methodsFor: 'error handling' stamp: 'JAAyer 3/24/2010 17:06'!
invokeDeprecated: aDeprecatedSelector withArguments: anOldArgumentArray orForwardTo: aNewSelector withArguments: aNewArgumentArray
	| parentContext |

	parentContext := thisContext sender.
	(self class includesSelector: aDeprecatedSelector)
		ifFalse: [
			aNewSelector ifNil: [^ self].
			^ self
				perform: aNewSelector
				withArguments: aNewArgumentArray].

	(Deprecation
		method: (self class lookupSelector: aDeprecatedSelector)
		explanation: 'use ', parentContext selector, ' instead'
		on: nil
		in: nil) signal.
	self
		perform: aDeprecatedSelector
		withArguments: anOldArgumentArray.! !

!SAXHandler methodsFor: 'testing' stamp: 'JAAyer 3/11/2010 06:43'!
isValidating
	self driver isValidating! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 06:45'!
isValidating: aBoolean
	self driver isValidating: aBoolean! !

!SAXHandler methodsFor: 'parsing' stamp: 'JAAyer 3/9/2010 01:32'!
parseDocument
	self startDocument.
	[self driver nextToken isNil or: [self eod]] whileFalse.

	"Result of parsing"
	^ self.! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:26'!
processingInstruction: piName data: dataString
	"This call corresonds to the Java SAX call
	processingInstruction(java.lang.String target, java.lang.String data)."! !

!SAXHandler methodsFor: 'entity' stamp: 'mir 8/11/2000 17:33'!
resolveEntity: publicID systemID: systemID
	"This call corresonds to the Java SAX call
	resolveEntity(java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
skippedEntity: anEntityName
	"This call corresonds to the Java SAX call
	skippedEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'lexical' stamp: 'mir 8/11/2000 18:53'!
startCData
	"This call corresponds to the Java SAX ext call
	startCData()."! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:21'!
startDTD: aRootElementName publicID: aPublicId systemID: aSystemId
	"This call corresponds to the Java SAX ext call
	startDTD(java.lang.String name, java.lang.String publicId, java.lang.String systemId)."! !

!SAXHandler methodsFor: 'content' stamp: 'mir 8/11/2000 16:45'!
startDocument
	"This call corresonds to the Java SAX call
	startDocument()."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 17:06'!
startElement: aQualifiedName attributes: aDictionary
	self
		invokeDeprecated: #startElement:attributeList:
		withArguments: (Array with: aQualifiedName with: aDictionary)
		orForwardTo: nil
		withArguments: nil.! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:47'!
startElement: aQualifiedName prefix: aPrefix uri: aUri attributes: aDictionary
	self startElement: aQualifiedName attributes: aDictionary! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 8/9/2010 17:20'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	"This call corresonds to the Java SAX call
	startElement(java.lang.String namespaceURI, java.lang.String localName,
		java.lang.String qName, Attributes atts)."

	self
		invokeDeprecated: #startElement:namespaceURI:namespace:attributeList:
		withArguments: (Array with: aLocalName with: aUri with: aPrefix with: aDictionary)
		orForwardTo: #startElement:prefix:uri:attributes:
		withArguments: (Array with: aQualifiedName with: aPrefix with: aUri with: aDictionary).! !

!SAXHandler methodsFor: 'lexical' stamp: 'JAAyer 11/10/2010 19:22'!
startEntity: anEntityName
	"This call corresponds to the Java SAX ext call
	startEntity(java.lang.String name)."! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/10/2010 19:22'!
startPrefixMapping: aPrefix uri: aUri
	"This call corresonds to the Java SAX call
	startPrefixMapping(java.lang.String prefix, java.lang.String uri)."! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 05:28'!
useNamespaces: aBoolean
	self deprecated: 'use #usesNamespaces: instead'.

	self driver usesNamespaces: aBoolean.! !

!SAXHandler methodsFor: 'testing' stamp: 'JAAyer 3/8/2010 10:38'!
usesNamespaces
	^ self driver usesNamespaces! !

!SAXHandler methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 05:59'!
usesNamespaces: aBoolean
	self driver usesNamespaces: aBoolean! !

!SAXHandler methodsFor: 'content' stamp: 'JAAyer 11/13/2010 21:09'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self
		invokeDeprecated: #documentAttributes:
		withArguments:
			(Array with: (Dictionary
				with: 'version' -> aVersionFloat asString
				with: 'encoding' -> anEncoding))
		orForwardTo: nil
		withArguments: nil! !

SAXHandler subclass: #XMLDOMParser
	instanceVariableNames: 'document nodeStack nodeFactory preservesCDataSections isInCDataSection'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLDOMParser commentStamp: 'JAAyer 3/23/2010 14:08' prior: 0!
XMLDOMParser parses XML into a tree of XML nodes representing the document structure. To see what kinds of nodes it produces and what can be done with them, browse XML-Parser-Nodes. !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 05:41'!
characters: aString
	(self preservesCDataSections and: [self isInCDataSection])
		ifTrue: [self topNode addNode: (self nodeFactory cdataClass string: aString)]
		ifFalse: [self topNode addContent: (self nodeFactory stringClass string: aString)]
! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/9/2010 01:38'!
document
	^ document! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 3/8/2010 20:31'!
document: aDocument
	document := aDocument! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'JAAyer 3/8/2010 20:29'!
domDocument
	[self parseDocument] ensure: [self driver stream close].

	^ document.! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 8/9/2010 17:27'!
endCData
	isInCDataSection := false! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 06:02'!
endDocument
	self popNode.
	super endDocument.! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 3/24/2010 16:55'!
endElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName
	self popNode! !

!XMLDOMParser methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 17:27'!
isInCDataSection
	^ isInCDataSection ifNil: [isInCDataSection := false]! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 05:40'!
nodeFactory
	^ nodeFactory ifNil: [nodeFactory := XMLNodeFactory new]! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 05:40'!
nodeFactory: aNodeFactory
	nodeFactory := aNodeFactory! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/22/2010 15:28'!
nodeStack
	^ nodeStack ifNil: [nodeStack := OrderedCollection new: 30]! !

!XMLDOMParser methodsFor: 'parsing' stamp: 'JAAyer 3/9/2010 01:30'!
parseDocument
	super parseDocument.
	^ self document.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
popNode
	^ self nodeStack removeLast! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 17:25'!
preservesCDataSections
	^ preservesCDataSections ifNil: [preservesCDataSections := false]! !

!XMLDOMParser methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 17:25'!
preservesCDataSections: aBoolean
	"Specify whether the receiver shoud preserve CDATA sections.
	If true XMLCDATASection nodes will be created. Otherwise plain
	XMLString nodes will be used."

	preservesCDataSections := aBoolean! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 05:42'!
processingInstruction: piName data: dataString
	self topNode addNode: (self nodeFactory piClass target: piName data: dataString)! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 11/5/2010 19:27'!
pushElement: anElement

	self nodeStack notEmpty
		ifTrue: [self topNode addElement: anElement].

	self pushNode: anElement.! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:31'!
pushNode: aNode
	^ self nodeStack addLast: aNode
! !

!XMLDOMParser methodsFor: 'lexical' stamp: 'JAAyer 8/9/2010 17:27'!
startCData
	isInCDataSection := true! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 05:43'!
startDocument
	self document: self nodeFactory documentClass new.
	self pushNode: self document ! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 05:43'!
startElement: aQualifiedName prefix: aPrefix uri: aUri localName: aLocalName attributes: aDictionary
	self pushElement:
		((self nodeFactory
			classForElement: aQualifiedName
			prefix: aPrefix
			uri: aUri
			localName: aLocalName)
				qualifiedName: aQualifiedName
				prefix: aPrefix
				localName: aLocalName
				attributes: aDictionary
				namespaceScope: self currentScope)! !

!XMLDOMParser methodsFor: 'private' stamp: 'JAAyer 3/16/2010 09:29'!
topNode
	^ self nodeStack isEmpty
		ifTrue: [nil]
		ifFalse: [self nodeStack last]! !

!XMLDOMParser methodsFor: 'content' stamp: 'JAAyer 11/10/2010 05:58'!
xmlVersion: aVersionFloat encoding: anEncoding standalone: aBoolean
	self document
		version: aVersionFloat;
		encoding: anEncoding! !

Object subclass: #XMLNamespaceScope
	instanceVariableNames: 'defaultNamespace prefixMappings'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScope commentStamp: 'JAAyer 3/11/2010 00:01' prior: 0!
An XMLNamespaceScope is a set of mappings from namespace prefixes to URIs, and also a default URI. Scopes can be created from other scopes by sending #enclosingScope: to XMLNamespaceScope with the other XMLNamespaceScope as the argument.!

!XMLNamespaceScope class methodsFor: 'instance creation' stamp: 'JAAyer 3/5/2010 09:50'!
enclosingScope: aScope
	^ self new inheritMappingsFrom: aScope! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:50'!
defaultNamespace
	^ defaultNamespace ifNil: [defaultNamespace := '']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:26'!
defaultNamespace: aUri
	aUri = self xmlNamespaceURI
		ifTrue: [XMLNamespaceException signal: 'Cannot use "xml" namespace URI as default namespace'].
	aUri = self xmlnsNamespaceURI
		ifTrue: [XMLNamespaceException signal: 'Cannot use "xmlns" namespace URI as default namespace'].

	defaultNamespace := aUri.! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/5/2010 22:30'!
ensureAttributePrefix: aPrefix localName: aLocalName hasNoAliasIn: aDictionary
	| namespaceUri aliasingAttributeName |

	namespaceUri := self resolvePrefix: aPrefix.

	(self prefixesAliasing: aPrefix) do: [:aliasingPrefix |
		aliasingAttributeName := aliasingPrefix, ':', aLocalName.
		(aDictionary includesKey: aliasingAttributeName)
			ifTrue: [
				self
					errorAttribute: aliasingAttributeName
					aliases: aPrefix, ':', aLocalName
					with: namespaceUri]].! !

!XMLNamespaceScope methodsFor: 'errors' stamp: 'JAAyer 6/20/2010 23:17'!
errorAttribute: anAttribute aliases: anAliasedAttribute with: aUri
	XMLNamespaceException signal:
		('Attribute "{1}" aliases attribute "{2}"; both prefixes map to {3}'
			format: (Array with: anAttribute with: anAliasedAttribute with: aUri))! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 20:10'!
hasDefaultNamespace
	^ defaultNamespace notNil and: [defaultNamespace notEmpty]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/5/2010 01:24'!
hasNamespaces
	^ self hasDefaultNamespace or: [self hasPrefixMappings]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 7/26/2010 18:17'!
hasPrefixMappings
	^ prefixMappings notNil and: [prefixMappings notEmpty]! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 11/6/2010 01:43'!
includesPrefix: aPrefix
	^ self prefixMappings includesKey: aPrefix! !

!XMLNamespaceScope methodsFor: 'testing' stamp: 'JAAyer 8/8/2010 14:43'!
includesPrefix: aPrefix mappedTo: aUri
	^ (self includesPrefix: aPrefix)
		and: [(self resolvePrefix: aPrefix) = aUri]! !

!XMLNamespaceScope methodsFor: 'initializing' stamp: 'JAAyer 10/16/2010 04:21'!
inheritMappingsFrom: anEnclosingScope
	anEnclosingScope hasNamespaces
		ifFalse: [^ self].

	self hasDefaultNamespace
		ifFalse: [self defaultNamespace: anEnclosingScope defaultNamespace].

	anEnclosingScope hasPrefixMappings
		ifTrue: [
			anEnclosingScope prefixesAndURIsDo: [:prefix :uri |
				(self includesPrefix: prefix)
					ifFalse: [self mapPrefix: prefix to: uri]]].! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/21/2010 09:28'!
mapPrefix: aPrefix to: aUri
	aPrefix isEmptyOrNil
		ifTrue: [^ self defaultNamespace: aUri].

	aPrefix = 'xml'
		ifTrue: [
			aUri = self xmlNamespaceURI
				ifFalse: [
					XMLNamespaceException signal:
						'Cannot rebind the "xml" prefix to a different namespace URI'].
			^ self].

	aPrefix = 'xmlns'
		ifTrue: [XMLNamespaceException signal: 'Cannot bind "xmlns" prefix at all'].

	(aUri = self xmlNamespaceURI or: [aUri = self xmlnsNamespaceURI])
		ifTrue: [XMLNamespaceException signal: 'Cannot alias the "xml" or "xmlns" prefixes'].

	self prefixMappings at: aPrefix put: (aUri ifNil: [''])! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/20/2010 08:03'!
prefixMappings
	^ prefixMappings ifNil: [prefixMappings := OrderPreservingDictionary new]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/5/2010 01:18'!
prefixes
	^ self prefixMappings keys! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 20:11'!
prefixesAliasing: aPrefix
	"Locate all prefixes that map to the same URI the given prefix does."
	| uri |
	
	uri := self resolvePrefix: aPrefix ifUnmapped: [^ #()].

	^ (self prefixesMappedTo: uri) copyWithout: aPrefix! !

!XMLNamespaceScope methodsFor: 'enumerating' stamp: 'JAAyer 8/5/2010 01:00'!
prefixesAndURIsDo: aTwoArgumentBlock
	self prefixMappings keysAndValuesDo: aTwoArgumentBlock! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 8/8/2010 14:43'!
prefixesMappedTo: aUri
	^ self prefixes select: [:each | self includesPrefix: each mappedTo: aUri]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:50'!
resolvePrefix: aPrefix
	"Retrieve the URI of the given namespace prefix, if it is defined."

	^ self resolvePrefix: aPrefix ifUnmapped: ['']! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:27'!
resolvePrefix: aPrefix ifUnmapped: aBlock
	aPrefix isEmptyOrNil
		ifTrue: [^ self defaultNamespace].
	aPrefix = 'xml'
		ifTrue: [^ self xmlNamespaceURI].
	aPrefix = 'xmlns'
		ifTrue: [^ self xmlnsNamespaceURI].

	^ self prefixMappings at: aPrefix ifAbsent: aBlock! !

!XMLNamespaceScope methodsFor: 'private' stamp: 'JAAyer 11/6/2010 02:22'!
validatePrefix: aPrefix of: aMarkupEntityType named: aName
	(aPrefix isEmptyOrNil or: [aPrefix = 'xml'])
		ifTrue: [^ self].

	aPrefix = 'xmlns'
		ifTrue: [
			XMLNamespaceException signal:
				('Cannot use reserved "xmlns" prefix for {1} {2}' format:
					(Array with: aMarkupEntityType with: aName))].

	(self includesPrefix: aPrefix)
		ifFalse: [
			XMLNamespaceException signal:
				('Unmapped prefix "{1}" in {2} {3}' format:
					(Array with: aPrefix with: aMarkupEntityType with: aName))].! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/6/2010 02:00'!
validatePrefix: aPrefix ofAttributeName: anAttributeName
	self validatePrefix: aPrefix of: 'attribute' named: anAttributeName! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/6/2010 02:00'!
validatePrefix: aPrefix ofElementName: anElementName
	self validatePrefix: aPrefix of: 'element' named: anElementName! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 11/5/2010 22:30'!
validatePrefixedAttributeName: anAttributeName in: aDictionary
	anAttributeName splitQualifiedNameInto: [:prefix :localName |
		prefix ifEmpty: [^ self].

		self
			validatePrefix: prefix
			ofAttributeName: anAttributeName.
		self
			ensureAttributePrefix: prefix
			localName: localName
			hasNoAliasIn: aDictionary]! !

!XMLNamespaceScope methodsFor: 'validation' stamp: 'JAAyer 8/6/2010 23:46'!
validatePrefixedAttributeNames: aDictionary
	aDictionary keysDo: [:attributeName |
		self
			validatePrefixedAttributeName: attributeName
			in: aDictionary]! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:25'!
xmlNamespaceURI
	^ 'http://www.w3.org/XML/1998/namespace'! !

!XMLNamespaceScope methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 20:25'!
xmlnsNamespaceURI
	^ 'http://www.w3.org/2000/xmlns/'! !

Object subclass: #XMLNamespaceScopeManager
	instanceVariableNames: 'scopes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Namespaces'!

!XMLNamespaceScopeManager commentStamp: '<historical>' prior: 0!
This class represents a stack of XMLNamespaceScopes. Sending #enterScope to an instance will cause it to push a new scope onto the stack that inherits all mappings from the previous, enclosing scope. Sending #leaveScop causes that scope to be removed.!

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 14:27'!
currentScope
	^ self scopes
		at: self scopes size
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeManager methodsFor: 'scope' stamp: 'JAAyer 8/8/2010 14:45'!
declarePrefix: aPrefix uri: aUri
	"Declare the given name space prefix with the given URL"

	self currentScope mapPrefix: aPrefix to: aUri! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/3/2010 22:37'!
defaultNamespace
	^ self currentScope defaultNamespace! !

!XMLNamespaceScopeManager methodsFor: 'private' stamp: 'JAAyer 3/13/2010 13:44'!
emptyScope
	^ XMLNamespaceScope new! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 22:14'!
enclosingScope
	^ self scopes
		at: self scopes size - 1
		ifAbsent: [self emptyScope]! !

!XMLNamespaceScopeManager methodsFor: 'scope' stamp: 'JAAyer 3/13/2010 13:42'!
enterScope
	self pushScope: (XMLNamespaceScope enclosingScope: self currentScope)! !

!XMLNamespaceScopeManager methodsFor: 'scope' stamp: 'JAAyer 7/27/2010 14:31'!
exitScope
	self popScope! !

!XMLNamespaceScopeManager methodsFor: 'testing' stamp: 'JAAyer 8/6/2010 12:07'!
includesPrefix: aPrefix
	^ self currentScope includesPrefix: aPrefix! !

!XMLNamespaceScopeManager methodsFor: 'testing' stamp: 'JAAyer 8/8/2010 14:43'!
includesPrefix: aPrefix mappedTo: aUri
	^ self currentScope includesPrefix: aPrefix mappedTo: aUri! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/4/2010 00:12'!
popScope
	self scopes removeLast! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/13/2010 13:43'!
pushScope: aScope
	self scopes add: aScope! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 06:00'!
resolvePrefix: aPrefix
	^ self currentScope resolvePrefix: aPrefix! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 09:50'!
resolvePrefix: aPrefix ifUnmapped: aBlock
	^ self currentScope resolvePrefix: aPrefix ifUnmapped: aBlock! !

!XMLNamespaceScopeManager methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 21:35'!
scopes
	^ scopes ifNil: [scopes := OrderedCollection new: 20]! !

!XMLNamespaceScopeManager methodsFor: 'validation' stamp: 'JAAyer 8/7/2010 13:42'!
validatePrefix: aPrefix ofElementName: anElementName
	self currentScope
		validatePrefix: aPrefix
		ofElementName: anElementName! !

!XMLNamespaceScopeManager methodsFor: 'validation' stamp: 'JAAyer 8/6/2010 23:14'!
validatePrefixedAttributeNames: aDictionary
	self currentScope validatePrefixedAttributeNames: aDictionary.! !

Object subclass: #XMLNestedStreamReader
	instanceVariableNames: 'stream nestedStreams hasNestedStreams buffer peekChar normalizedLineEndingChar currentLineNumber'
	classVariableNames: 'CarriageReturn HexDigitCharacterMap LineFeed SeparatorCharacters'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNestedStreamReader commentStamp: '<historical>' prior: 0!
This class represents a stream reader that implements pushBack: using nested streams. This enables subsitution/replacement to be performed without modifying the underlying collections streamed-over or having to copy them. It also performs line-ending normalization, transforming CR and CRLF sequences into a single LF character.

Each nested stream is wrapped in some kind of XMLMinimalReadStream, which implements through forwarding the minimum stream protocol this class expects.

(The code in this class is optimized; refactor with care.)!

!XMLNestedStreamReader class methodsFor: 'initialization' stamp: 'JAAyer 10/16/2010 12:05'!
initialize
	"self initialize"

	SeparatorCharacters := CharacterSet newFrom: Character separators.

	CarriageReturn := Character cr.
	LineFeed := Character lf.

	HexDigitCharacterMap :=
		CharacterMap newFrom:
			(($0 to: $9), ($a to: $f), ($A to: $F)
				collect: [:each | each -> each asUppercase digitValue]).! !

!XMLNestedStreamReader class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 09:34'!
on: anInitialStream
	^ self new stream: anInitialStream! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'JAAyer 10/20/2010 11:49'!
atEnd
	^ peekChar isNil and: [hasNestedStreams not and: [stream atEnd]]! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/20/2010 11:38'!
basicNext
	"Returns next character in the stream after performing line-ending normalization.
	Normalization does not occur across nested streams."

	| nextChar |

	stream atEnd
		ifTrue: [
			hasNestedStreams ifFalse: [^nil].
			self popCurrentStream.
			[hasNestedStreams and: [stream atEnd]]
				whileTrue: [self popCurrentStream].
			stream atEnd
				ifTrue: [^ nil]].

	(nextChar := stream next) == CarriageReturn
		ifTrue: [
			nextChar := normalizedLineEndingChar.
			(stream peek == LineFeed)
				ifTrue: [stream next]]
		ifFalse: [
			(nextChar == LineFeed)
				ifTrue: [nextChar := normalizedLineEndingChar]].

	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/14/2010 00:39'!
currentLineNumber
	^ currentLineNumber! !

!XMLNestedStreamReader methodsFor: 'initialization' stamp: 'JAAyer 10/18/2010 16:07'!
initialize
	stream := nil.
	nestedStreams := nil.
	hasNestedStreams := false.
	peekChar := nil.
	buffer := WriteStream on: (String new: 128).
	currentLineNumber := 1.
	normalizedLineEndingChar := LineFeed.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/18/2010 16:17'!
next
	"Return the next character from the current input stream. If the current
	stream is at end, pop to the next nesting level if there is one."

	| nextChar |

	peekChar
		ifNil: [nextChar := self basicNext]
		ifNotNil: [
			nextChar := peekChar.
			peekChar := nil].
	nextChar == normalizedLineEndingChar
		ifTrue: [currentLineNumber := currentLineNumber + 1].
	^ nextChar.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 14:53'!
nextDecimalInteger
	| digitChar |

	buffer reset.
	[self atEnd not and: [(digitChar := self peek) isDigit]]
		whileTrue: [buffer nextPut: self next].
	^ Integer readFrom: buffer contents readStream.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 9/7/2010 10:09'!
nextHexInteger
	| maxBaseValue value digitChar |

	maxBaseValue := SmallInteger maxVal / 16.
	value := HexDigitCharacterMap at: self peek ifAbsent: [^ 0].
	self next.
	[self atEnd not
		and: [HexDigitCharacterMap includesKey: (digitChar := self peek)]]
		whileTrue: [| baseValue digitValue |
			self next.

			value <= maxBaseValue
				ifFalse: [^ value].
			baseValue := value * 16.

			digitValue := HexDigitCharacterMap at: digitChar.
			((SmallInteger maxVal - digitValue) > baseValue)
				ifFalse: [^ value].
			value := baseValue + digitValue].
	^ value.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 8/31/2010 14:26'!
nextIntegerWithBase: aBase
	aBase = 10
		ifTrue: [^ self nextDecimalInteger].
	aBase = 16
		ifTrue: [^ self nextHexInteger].

	self error: 'Only base 10 and 16 supported'.! !

!XMLNestedStreamReader methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 14:46'!
nextMatchAll: aString
	| i oldStream oldPosition oldPeekChar nextChar pushBackString |

	(oldPeekChar := self peek) == (aString at: 1)
		ifFalse: [^ false].
	oldPosition := (oldStream := stream) position.
	i := 1.
	[(i <= aString size)
		and: [(aString at: i) == (nextChar := self next)]]
		whileTrue: [i := i + 1].
	(i > aString size)
		ifTrue: [^ true].

	stream == oldStream
		ifTrue: [
			peekChar := oldPeekChar.
			stream position: oldPosition]
		ifFalse:[
			pushBackString := aString copyFrom: 1 to: i - 1.
			self pushBack:
				(nextChar
					ifNotNil: [pushBackString copyWith: nextChar]
					ifNil: [pushBackString])].
	^ false.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 11:39'!
nextWhitespace
	| nextChar |

	buffer reset.
	[(nextChar := self peek) notNil
		and: [SeparatorCharacters includes: nextChar]]
		whileTrue: [buffer nextPut: self next].
	^ buffer stringContents.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/18/2010 16:17'!
peek
	"Return the next character from the current input stream."

	^ peekChar ifNil: [peekChar := self basicNext]! !

!XMLNestedStreamReader methodsFor: 'private' stamp: 'JAAyer 10/18/2010 16:08'!
popCurrentStream
	stream close.
	stream := nestedStreams removeLast.
	nestedStreams ifEmpty: [hasNestedStreams := false].! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 10/16/2010 04:45'!
pushBack: aString
	"Fixed to push the string before the peek char (if any)."
	| pushBackString |

	pushBackString :=
		peekChar
			ifNil: [aString]
			ifNotNil: [aString copyWith: peekChar].
	peekChar := nil.
	self pushStream: (ReadStream on: aString)! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 10/18/2010 16:08'!
pushStream: aStream
	"Continue parsing from the new nested stream."
	self unpeek.

	nestedStreams
		ifNil: [nestedStreams := OrderedCollection with: stream]
		ifNotNil: [nestedStreams addLast: stream].
	hasNestedStreams := true.

	self stream: aStream.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 7/21/2010 11:39'!
skipSeparators
	| nextChar |

	[(nextChar := self peek) notNil
		and: [SeparatorCharacters includes: nextChar]]
		whileTrue: [self next].! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 7/12/2010 01:12'!
skipUpTo: aDelimiter
	| nextChar |

	[self atEnd or: [(nextChar := self next) == aDelimiter]] whileFalse.
	^ nextChar == aDelimiter.! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 08:01'!
stream
	^ stream! !

!XMLNestedStreamReader methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 14:44'!
stream: aNewStream
	stream := aNewStream! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 10/15/2010 14:51'!
streamEncoding: aCharacterEncodingName
	| converterClass textConverterBase |

	(stream respondsTo: #converter) ifFalse: [^ self].
	textConverterBase := Smalltalk at: #TextConverter ifAbsent: [^ self].
	converterClass :=
		textConverterBase defaultConverterClassForEncoding:
			aCharacterEncodingName asLowercase.
	converterClass
		ifNotNil: [stream converter: converterClass new].! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 3/11/2010 07:46'!
unpeek
	"Fixed to use nested stream since multi-byte streams
	do not properly override pushBack: to deal with multi-byte
	characters."
	
	peekChar ifNotNil: [self pushBack: '']! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 8/11/2010 14:03'!
upTo: aDelimiter
	| nextChar |

	buffer reset.
	[self atEnd or: [(nextChar := self next) == aDelimiter]]
		whileFalse: [buffer nextPut: nextChar].

	^ buffer stringContents.! !

!XMLNestedStreamReader methodsFor: 'streaming' stamp: 'JAAyer 3/19/2010 14:27'!
upToAll: aDelimitingString
	"Answer a subcollection from the current access position to the occurrence
	(if any, but not inclusive) of delimitingString. If delimitingString is not
	in the stream, answer the entire rest of the stream."

	buffer reset.
	[self atEnd or: [self nextMatchAll: aDelimitingString]]
		whileFalse: [buffer nextPut: self next].
	^ buffer stringContents.! !

Object subclass: #XMLNestedStreamWriter
	instanceVariableNames: 'streams nextStreamIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLNestedStreamWriter commentStamp: 'JAAyer 7/12/2010 01:22' prior: 0!
This class handles the allocation and reuse of multiple WriteStreams. It uses a single execute-around method, #writeWith:, which takes a block that it evaluates with a ready-to-use WriteStream.
!

!XMLNestedStreamWriter class methodsFor: 'instance creation' stamp: 'JAAyer 9/19/2010 23:48'!
new
	^ self new: 5! !

!XMLNestedStreamWriter class methodsFor: 'instance creation' stamp: 'JAAyer 9/19/2010 23:51'!
new: aNumberOfStreams
	^ self basicNew initialize: aNumberOfStreams! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 9/19/2010 23:54'!
acquireStream
	| stream |

	(nextStreamIndex <= streams size)
		 ifFalse: [self grow].

	(stream := streams at: nextStreamIndex) reset.
	nextStreamIndex := nextStreamIndex + 1.
	^ stream.! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 10/16/2010 04:41'!
grow
	streams :=
		(Array new: ((streams size * 2) max: 10))
			replaceFrom: 1
			to: streams size
			with: streams
			startingAt: 1.
	nextStreamIndex to: streams size do: [:i | streams at: i put: self newStream].! !

!XMLNestedStreamWriter methodsFor: 'initialization' stamp: 'JAAyer 9/19/2010 23:50'!
initialize: aNumberOfStreams
	streams := Array new: aNumberOfStreams.
	1 to: aNumberOfStreams do: [:i | streams at: i put: self newStream].
	nextStreamIndex := 1.! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 7/12/2010 01:19'!
newStream
	^ WriteStream on: (String new: 128)! !

!XMLNestedStreamWriter methodsFor: 'private' stamp: 'JAAyer 9/19/2010 23:54'!
releaseStream
	nextStreamIndex := nextStreamIndex - 1! !

!XMLNestedStreamWriter methodsFor: 'writing' stamp: 'JAAyer 9/19/2010 23:42'!
writeWith: aBlock
	| result |

	"Avoid ensure: for performance"
	result := aBlock value: self acquireStream.
	self releaseStream.
	^ result.! !

Object subclass: #XMLNode
	uses: TXMLObservable
	instanceVariableNames: 'parent notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNode commentStamp: 'JAAyer 7/10/2010 05:48' prior: 0!
This is a base class for XML nodes. It contains testing messages and methods to set and access the parent node.!

XMLNode class
	uses: TXMLObservable classTrait
	instanceVariableNames: ''!

XMLNode subclass: #XMLAttribute
	instanceVariableNames: 'element name value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLAttribute commentStamp: '<historical>' prior: 0!
The class represents an attribute node, to be stored in an XMLAttributeList. You generally won't need to interact with these objects directly; XMLElement's various #attribute- messages will normally suffice. The name of an attribute node can be namespaced using prefixes; however, the prefix must be mapped to a namespace URI in the containing element node. Unprefixed attributes have no namespace, not even the default one, per the XML namespaces standard.

Note that attribute nodes are not children of the elements that contain them; rather, they are considered to be "part" of the element, and this relationship is modeled using a separate instance variable ("element") with different accessor messages.!

!XMLAttribute class methodsFor: 'instance creation' stamp: 'JAAyer 10/24/2010 18:05'!
name: aName value: aValue
	^ self new
		name: aName;
		value: aValue! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 18:04'!
element
	^ element! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 18:04'!
element: anElement
	element := anElement! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 3/7/2010 04:38'!
isAttribute
	^ true! !

!XMLAttribute methodsFor: 'testing' stamp: 'JAAyer 7/22/2010 08:29'!
isNamed: aName
	^ self nodeName isSameAs: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
name: aName
	self nodeName qualified: aName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 22:02'!
namespaceURI
	(self element notNil and: [self hasPrefix])
		ifFalse: [^ ''].
	
	^ self element resolvePrefix: self prefix.! !

!XMLAttribute methodsFor: 'private' stamp: 'JAAyer 7/27/2010 12:52'!
nodeName
	^ name ifNil: [
		(name := XMLNodeName new)
			addObserver: self;
			yourself]! !

!XMLAttribute methodsFor: 'notifications' stamp: 'JAAyer 7/11/2010 02:25'!
nodeNameChangedFrom: anOldName to: aNewName
	self
		notifyOf: #renamedAttribute:from:to:
		with: self
		with: anOldName
		with: aNewName! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 6/15/2010 15:15'!
prefix: aPrefix
	self nodeName prefix: aPrefix! !

!XMLAttribute methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter writeName: self name value: self value! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
value
	^ value ifNil: [value := '']! !

!XMLAttribute methodsFor: 'accessing' stamp: 'JAAyer 3/5/2010 08:18'!
value: aString
	value := aString! !

XMLNode subclass: #XMLComment
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLComment commentStamp: '<historical>' prior: 0!
This class represents comments preserved as nodes. !

!XMLComment class methodsFor: 'instance creation' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	^ self new string: aString! !

!XMLComment methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ true! !

!XMLComment methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter comment: self string! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string
	^ string ifNil: [string := '']! !

!XMLComment methodsFor: 'accessing' stamp: 'JAAyer 8/11/2010 22:14'!
string: aString
	string := aString! !

XMLNode subclass: #XMLDeclarationNode
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDeclarationNode commentStamp: 'JAAyer 8/11/2010 21:03' prior: 0!
This is a generic class to represent markup declarations in the internal subset. Each subclass represents a specific kind of declaration, and each is immutable.!

XMLDeclarationNode subclass: #XMLAttributeDeclarationNode
	instanceVariableNames: 'attributeName type defaultDeclaration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLAttributeDeclarationNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 8/9/2010 19:57'!
elementName: anElementName attributeName: anAttributeName type: aType defaultDeclaration: aDefaultDeclaration
	^ self new
		setElementName: anElementName
		attributeName: anAttributeName
		type: aType
		defaultDeclaration: aDefaultDeclaration! !

!XMLAttributeDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
attributeName
	^ attributeName ifNil: [attributeName := '']! !

!XMLAttributeDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
defaultDeclaration
	^ defaultDeclaration ifNil: [defaultDeclaration := '']! !

!XMLAttributeDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:48'!
elementName
	^ self name! !

!XMLAttributeDeclarationNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter attributeDeclaration
		elementName: self elementName;
		attributeName: self attributeName;
		type: self type;
		defaultDeclaration: self defaultDeclaration! !

!XMLAttributeDeclarationNode methodsFor: 'initialization' stamp: 'JAAyer 8/9/2010 19:59'!
setElementName: anElementName attributeName: anAttributeName type: aType defaultDeclaration: aDefaultDeclaration
	name := anElementName.
	attributeName := anAttributeName.
	type := aType.
	defaultDeclaration := aDefaultDeclaration.! !

!XMLAttributeDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:42'!
type
	^ type ifNil: [type := '']! !

!XMLDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:44'!
name
	^ name ifNil: [name := '']! !

XMLDeclarationNode subclass: #XMLElementDeclarationNode
	instanceVariableNames: 'contentModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLElementDeclarationNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:40'!
name: aName contentModel: aContentModelString
	^ self new setName: aName contentModel: aContentModelString! !

!XMLElementDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
contentModel
	^ contentModel ifNil: [contentModel := '']! !

!XMLElementDeclarationNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter elementDeclaration
		name: self name;
		contentModel: self contentModel! !

!XMLElementDeclarationNode methodsFor: 'initialization' stamp: 'JAAyer 7/20/2010 10:41'!
setName: aName contentModel: aContentModelString
	name := aName.
	contentModel := aContentModelString.! !

XMLDeclarationNode subclass: #XMLEntityDeclarationNode
	instanceVariableNames: 'replacement publicId systemId ndata'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLEntityDeclarationNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:53'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: nil! !

!XMLEntityDeclarationNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:53'!
name: aName publicID: aPublicId systemID: aSystemId ndata: aNotationName
	^ self new
		setName: aName
		replacement: nil
		publicID: aPublicId
		systemID: aSystemId
		ndata: aNotationName! !

!XMLEntityDeclarationNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 7/20/2010 10:52'!
name: aName replacement: aReplacement
	^ self new
		setName: aName
		replacement: aReplacement
		publicID: nil
		systemID: nil
		ndata: nil! !

!XMLEntityDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
ndata
	^ ndata ifNil: [ndata := '']! !

!XMLEntityDeclarationNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter entityDeclaration
		name: self name;
		replacement: self replacement;
		publicID: self publicID;
		systemID: self systemID;
		ndata: self ndata! !

!XMLEntityDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLEntityDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
replacement
	^ replacement ifNil: [replacement := '']! !

!XMLEntityDeclarationNode methodsFor: 'initialization' stamp: 'JAAyer 7/20/2010 10:51'!
setName: aName replacement: aReplacement publicID: aPublicId systemID: aSystemId ndata: aNotationName
	name := aName.
	replacement := aReplacement.
	publicId := aPublicId.
	systemId := aSystemId.
	ndata := aNotationName.! !

!XMLEntityDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

XMLDeclarationNode subclass: #XMLNotationDeclarationNode
	instanceVariableNames: 'publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNotationDeclarationNode classSide methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 00:55'!
name: aName publicID: aPublicId systemID: aSystemId
	^ self new setName: aName publicID: aPublicId systemID: aSystemId! !

!XMLNotationDeclarationNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter notationDeclaration
		name: self name;
		publicID: self publicID;
		systemID: self systemID! !

!XMLNotationDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
publicID
	^ publicId ifNil: [publicId := '']! !

!XMLNotationDeclarationNode methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 00:54'!
setName: aName publicID: aPublicId systemID: aSystemId
	name := aName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLNotationDeclarationNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:43'!
systemID
	^ systemId ifNil: [systemId := '']! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:15'!
addContent: aStringOrStringNode
	XMLDOMException signal: 'Node cannot contain content'! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:55'!
canonicallyPrintOn: aStream
	(XMLWriter on: aStream)
		isCanonical: true;
		writeWith: [:writer | self printXMLOn: writer]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:19'!
canonicallyPrinted
	^ String streamContents: [:stream | self printCanonicalOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:15'!
characterData
	self deprecated: 'use #contentString instead'.

	^ self contentString.! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 11/6/2010 01:21'!
contentString
	^ String streamContents: [:stream | self printContentOn: stream]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
hasChildren
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/25/2010 14:37'!
hasParent
	^ self parent notNil! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 3/7/2010 04:38'!
isAttribute
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:15'!
isComment
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/30/2010 19:19'!
isContentNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:50'!
isDocument
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
isEmpty
	self deprecated: 'use "hasChildren not" instead'.

	^ self hasChildren not! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 6/27/2010 19:31'!
isProcessingInstruction
	self deprecated: 'use #isPI instead'.

	^ self isPI.! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:33'!
isStringNode
	^ false! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 6/27/2010 19:31'!
isTag
	self deprecated: 'use #isElement instead'.

	^ self isElement.! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 19:01'!
isTagOrText
	self deprecated: 'use #isContentNode instead'.

	^ self isContentNode.! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 6/27/2010 19:31'!
isText
	self deprecated: 'use #isStringNode instead'.

	^ self isStringNode.! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextNode
	^ self parent ifNotNil: [self parent nodeAfter: self]! !

!XMLNode methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:49'!
notEmpty
	self deprecated: 'use #hasChildren instead'.

	^ self hasChildren.! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 6/11/2010 10:17'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent
	^ parent! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
parent: aParentNode
	parent := aParentNode! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/15/2010 23:23'!
prettyPrintOn: aStream
	(XMLWriter on: aStream)
		enablePrettyPrinting;
		writeWith: [:writer | self printXMLOn: writer]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:18'!
prettyPrinted
	^ String streamContents: [:stream | self prettyPrintOn: stream]! !

!XMLNode methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
previousNode
	^ self parent ifNotNil: [self parent nodeBefore: self]! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/6/2010 01:25'!
printCanonicalOn: aStream
	self deprecated: 'use #canonicallyPrintOn: instead'.

	self canonicallyPrintOn: aStream.! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 10/20/2010 19:11'!
printContentOn: aStream! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:54'!
printOn: aStream
	(XMLWriter on: aStream) writeWith: [:writer | self printXMLOn: writer].! !

!XMLNode methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	self subclassResponsibility! !

XMLNode subclass: #XMLNodeWithChildren
	instanceVariableNames: 'nodes'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeWithChildren commentStamp: '<historical>' prior: 0!
This class represents a node that can contain child nodes. It provides message that allow you to add, remove, access, and enumerate child nodes. The nodes themselves are stored in some type of XMLObservableList, the exact type of which varies depending on the subclass, and can be accessed directly by sending #nodes.!

XMLNodeWithChildren subclass: #XMLDoctypeDeclaration
	instanceVariableNames: 'root publicId systemId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDoctypeDeclaration commentStamp: 'JAAyer 7/21/2010 12:33' prior: 0!
This class represents the <!!DOCTYPE ...> declaration one often finds at the top of an XML document. It stores the root element name, formal public identifier (FPI) if any, system identifier (a URI), and then any nodes comprising the internal subset.!

!XMLDoctypeDeclaration class methodsFor: 'instance creation' stamp: 'JAAyer 7/21/2010 07:47'!
root: aRootElementName publicID: aPublicId systemID: aSystemId
	^ self new
		setRoot: aRootElementName
		publicID: aPublicId
		systemID: aSystemId! !

!XMLDoctypeDeclaration methodsFor: 'private' stamp: 'JAAyer 7/20/2010 10:26'!
nodesClass
	^ XMLInternalSubsetList! !

!XMLDoctypeDeclaration methodsFor: 'printing' stamp: 'JAAyer 11/8/2010 18:46'!
printXMLOn: aWriter
	aWriter doctype
		root: self root;
		publicID: self publicID;
		systemID: self systemID;
		withInternalSubset: [super printXMLOn: aWriter]! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 6/17/2010 21:35'!
publicID
	^ publicId! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 07:46'!
root
	^ root! !

!XMLDoctypeDeclaration methodsFor: 'initialization' stamp: 'JAAyer 7/21/2010 07:45'!
setRoot: aRootElementName publicID: aPublicId systemID: aSystemId
	root := aRootElementName.
	publicId := aPublicId.
	systemId := aSystemId.! !

!XMLDoctypeDeclaration methodsFor: 'accessing' stamp: 'JAAyer 6/17/2010 21:35'!
systemID
	^ systemId! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNode: aNode
	^ self nodes addLast: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode after: aPriorNode
	^ self nodes add: aNode after: aPriorNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 8/14/2010 21:05'!
addNode: aNode before: aSubsequentNode
	^ self nodes add: aNode before: aSubsequentNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:22'!
addNodeFirst: aNode
	^ self nodes addFirst: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:12'!
addNodes: aNodeCollection
	^ self nodes addAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:09'!
addedNode: aNode
	aNode
		addObserver: self;
		parent: self! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:55'!
firstNode
	^ self nodes first! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:48'!
hasChildren
	"Direct variable access avoids possibly unnecessary lazy initialization of the node list"

	^ nodes notNil and: [nodes notEmpty]! !

!XMLNodeWithChildren methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:23'!
includesNode: aNode
	^ self nodes includes: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:25'!
lastNode
	^ self nodes last! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:26'!
nodeAfter: aNode
	^ self nodes after: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
nodeBefore: aNode
	^ self nodes before: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 12:31'!
nodes
	^ nodes ifNil: [
		(nodes := self nodesClass new)
			addObserver: self;
			yourself]! !

!XMLNodeWithChildren methodsFor: 'private' stamp: 'JAAyer 8/6/2010 23:53'!
nodesClass
	self subclassResponsibility! !

!XMLNodeWithChildren methodsFor: 'enumerating' stamp: 'JAAyer 7/20/2010 10:27'!
nodesDo: aBlock
	self nodes do: aBlock! !

!XMLNodeWithChildren methodsFor: 'printing' stamp: 'JAAyer 11/8/2010 18:47'!
printXMLOn: writer
	self nodesDo: [:each | each printXMLOn: writer].! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNode: aNode
	^ self nodes remove: aNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes
	self nodes removeAll! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:27'!
removeNodes: aNodeCollection
	^ self nodes removeAll: aNodeCollection! !

!XMLNodeWithChildren methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:08'!
removedNode: aNode
	aNode
		removeObserver: self;
		parent: nil.! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 11/5/2010 22:39'!
replaceNode: aNode with: aReplacementNode
	^ self nodes
		at: (self nodes indexOf: aNode)
		put: aReplacementNode! !

!XMLNodeWithChildren methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:25'!
topNode
	self deprecated: 'use #firstNode instead'.

	^ self firstNode.! !

XMLNodeWithChildren subclass: #XMLNodeWithElements
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeWithElements commentStamp: '<historical>' prior: 0!
This class represents a node with elements. You will likely use its API through XMLDocument or XMLElement objects.

To retrieve child elements by name, you can send any of the #element- messages under "accessing." These messages accept both qualified and unqualified names and have constant time complexity. The #firstTag- messages search the receiver and its descendants using depth-first traversal for specific elements, and the #tagsNamed- messages iterate over descendent elements also using depth-first traversal.!

XMLNodeWithElements subclass: #XMLDocument
	instanceVariableNames: 'version encoding doctypeDeclaration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLDocument commentStamp: '<historical>' prior: 0!
This class represents a document node, which is often the root of an XMLNode tree.!

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration
	^ doctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/10/2010 05:16'!
doctypeDeclaration: aDoctypeDeclaration
	doctypeDeclaration := aDoctypeDeclaration! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:19'!
document
	self! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:03'!
encoding	
	^ encoding ifNil: [encoding := 'UTF-8']! !

!XMLDocument methodsFor: 'accessing' stamp: 'mir 1/17/2002 12:57'!
encoding: aString	
	encoding := aString! !

!XMLDocument methodsFor: 'testing' stamp: 'JAAyer 2/19/2010 23:51'!
isDocument
	^ true! !

!XMLDocument methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	self version ifNotNil: [
		aWriter xml
			version: self version;
			encoding: self encoding].
	self doctypeDeclaration
		ifNotNil: [self doctypeDeclaration printXMLOn: aWriter].
	
	super printXMLOn: aWriter! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 01:36'!
root
	^ self firstElement ! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 01:48'!
version	
	^ version! !

!XMLDocument methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 19:39'!
version: aVersionNumber
	version := aVersionNumber! !

XMLNodeWithElements subclass: #XMLElement
	instanceVariableNames: 'name attributes namespaceScope document'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLElement commentStamp: '<historical>' prior: 0!
The class represents an XML element node. It can have a name, attributes, associated namespace declarations and any number child nodes and elements.

The #attribute- messages provide a Dictionary-like prototcol for manipulating attribute nodes. The underlying XMLAttributeList can be accessed using #attributeNodes, and an order-preserving dictionary of attribute names/values can be obtained using #attributes. Namespaces can be declared using #declareNamespace:uri: or simultaniously with changing the element's name using #name:namespaceURI:. The default namespace can be accessed through #defaultNamespace and #defaultNamespace:.

See the comment for the superclass for how to access child nodes and elements.!

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/30/2010 22:09'!
name: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:05'!
name: aLocalName attributes: aDictionaryOrArray
	^ (self named: aLocalName)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri
	^ self new name: aQualifiedName namespaceURI: aUri! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaceURI: aUri attributes: aDictionaryOrArray
	^ (self name: aQualifiedName namespaceURI: aUri)
		setAttributes: aDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray
	^ self new
		declareNamespaces: aNamespaceDictionaryOrArray;
		name: aQualifiedName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 7/22/2010 03:06'!
name: aQualifiedName namespaces: aNamespaceDictionaryOrArray attributes: anAttributeDictionaryOrArray
	^ (self
		name: aQualifiedName
		namespaces: aNamespaceDictionaryOrArray)
			setAttributes: anAttributeDictionaryOrArray! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 3/11/2010 23:10'!
named: aLocalName
	^ self new name: aLocalName! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 10/24/2010 16:15'!
named: aName attributes: aDictionary
	self deprecated: 'use #name:attributes: instead'.

	^ self name: aName attributes: aDictionary.! !

!XMLElement class methodsFor: 'instance creation' stamp: 'JAAyer 9/2/2010 04:54'!
named: aLocalName namespace: aPrefix uri: aUri attributes: aDictionary
	self deprecated: 'use #name:namespaceURI:attributes: with a qualified name instead'.

	^ self
		name: aPrefix, ':', aLocalName
		namespaceURI: aUri
		attributes: aDictionary.! !

!XMLElement class methodsFor: 'private' stamp: 'JAAyer 6/25/2010 22:47'!
qualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary namespaceScope: aNamespaceScope
	^ self new
		setQualifiedName: aQualifiedName
		prefix: aPrefix
		localName: aLocalName
		attributes: anAttributeDictionary
		namespaceScope: aNamespaceScope! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
@ aName
	^ self attributeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 19:07'!
addContent: aStringOrStringNode
	"Adds a string or string node. If the last child node is already a string node,
	the characters being added are concatenated to it."

	(self hasChildren and: [self lastNode isStringNode])
		ifTrue: [self lastNode addContent: aStringOrStringNode]
		ifFalse: [self addNode: aStringOrStringNode asXMLString]! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:07'!
addedAttribute: anAttribute
	anAttribute
		addObserver: self;
		element: self.
	self validateAttributeName: anAttribute name.! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/20/2010 19:38'!
addedElement: anElement
	super addedElement: anElement.
	self hasNamespaces
		ifTrue: [anElement inheritMappingsFrom: self namespaceScope]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 13:52'!
at: aName
	self deprecated: 'use #attributeAt: or @ instead'.

	^ self attributeAt: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:40'!
at: aName ifAbsent: aBlock	
	self deprecated: 'use #attributeAt:ifAbsent: instead'.

	^ self attributeAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:40'!
at: aName ifAbsentPut: aBlock
	self deprecated: 'use #attributeAt:ifAbsentPut: instead'.

	^ self attributeAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:41'!
at: aName put: aValue
	self deprecated: 'use #attributeAt:put: instead'.

	^ self attributeAt: aName put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:15'!
attributeAssociations
	^ self attributeNodes asNameValueAssociations! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName
	^ self attributeAt: aName ifAbsent: ['']! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:33'!
attributeAt: aName ifAbsentPut: aBlock
	^ self attributeNodes attributeValueAt: aName ifAbsentPut: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:34'!
attributeAt: aName put: aValue
	^ self attributeNodes attributeValueAt: aName put: aValue! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:08'!
attributeNames
	^ self attributeNodes attributeNames! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:14'!
attributeNamesAndValuesDo: aTwoArgumentBlock
	self attributeNodes attributeNamesAndValuesDo: aTwoArgumentBlock! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/25/2010 22:09'!
attributeNamesDo: aBlock
	self attributeNodes attributeNamesDo: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 11:37'!
attributeNodeAt: aName
	^ self attributeNodes attributeNamed: aName! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:15'!
attributeNodeAt: aName ifAbsent: aBlock
	^ self attributeNodes attributeNamed: aName ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 09:08'!
attributeNodeAt: aLocalName namespaceURI: aUri
	^ self attributeNodes attributeNamed: aLocalName namespaceURI: aUri! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 09:08'!
attributeNodeAt: aLocalName namespaceURI: aUri ifAbsent: aBlock
	^ self attributeNodes attributeNamed: aLocalName namespaceURI: aUri ifAbsent: aBlock! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 12:41'!
attributeNodes
	^ attributes ifNil: [
		(attributes := XMLAttributeList new)
			addObserver: self;
			yourself]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/25/2010 22:16'!
attributes
	^ self attributeNodes asDictionary! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
contentString: aString
	self
		removeNodes;
		addContent: aString! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 19:31'!
contents
	self deprecated: 'use #stringNodes instead'.

	^ self stringNodes.! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/27/2010 19:31'!
contentsDo: aBlock
	self deprecated: 'use #stringNodesDo: instead'.

	self stringNodesDo: aBlock.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 11/5/2010 20:44'!
declareNamespaces: aDictionaryOrArray
	aDictionaryOrArray associationsDo: [:each |
		each key isEmptyOrNil
			ifTrue: [self defaultNamespace: each value]
			ifFalse: [self declarePrefix: each key uri: each value]]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 8/8/2010 14:45'!
declarePrefix: aPrefix uri: aUri
	(self namespaceScope includesPrefix: aPrefix mappedTo: aUri)
		ifTrue: [^ self].

	self mapPrefix: aPrefix to: aUri.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
defaultNamespace
	^ self namespaceScope defaultNamespace! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/12/2010 11:25'!
defaultNamespace: aNamespaceUri
	| oldDefaultNamespace |

	oldDefaultNamespace := self defaultNamespace.
	self
		updateEnclosedScopesWith: [:each | each defaultNamespace: aNamespaceUri]
		where: [:each | each defaultNamespace = oldDefaultNamespace].! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/12/2010 10:46'!
document
	^ document! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:12'!
document: aDocument
	document := aDocument! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 19:31'!
elementsAndContents
	self deprecated: 'use #contentNodes instead'.

	^ self contentNodes.! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 6/27/2010 19:31'!
elementsAndContentsDo: aBlock
	self deprecated: 'use contentNodesDo: instead'.

	self contentNodesDo: aBlock.! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 4/12/2010 00:37'!
expandedName
	^ self nodeName expandWith: self namespaceURI! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 3/11/2010 23:19'!
hasAttributes
	^ self attributeNodes notEmpty! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 20:32'!
hasContents
	self deprecated: 'use #hasStringNodes instead'.

	^ self hasStringNodes.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 10/16/2010 06:07'!
hasNamespaces
	^ namespaceScope notNil and: [namespaceScope hasNamespaces]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 6/26/2010 07:31'!
hasPrefix
	^ self nodeName isPrefixed! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 8/10/2010 04:11'!
hasStringNodes
	^ self nodes anySatisfy: [:each | each isStringNode]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 16:15'!
includesAttribute: aName
	^ self attributeNodes includesAttributeNamed: aName! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:09'!
inheritMappingsFrom: aScope
	self recurse: [:each | each namespaceScope inheritMappingsFrom: aScope]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 4/11/2010 22:59'!
isElement
	^ true! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 16:15'!
isNamed: aName
	^ self nodeName isSameAs: aName! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 16:15'!
isNamedAny: aNameArray
	^ aNameArray anySatisfy: [:each | self isNamed: each]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:16'!
localName
	^ self nodeName localPart! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/8/2010 14:43'!
mapPrefix: aPrefix to: aUri
	| oldMapping |

	oldMapping :=
		self namespaceScope
			resolvePrefix: aPrefix
			ifUnmapped: [nil].
	self
		updateEnclosedScopesWith: [:each | each mapPrefix: aPrefix to: aUri]
		where: [:each |
			(each includesPrefix: aPrefix mappedTo: oldMapping)
				or: [(each includesPrefix: aPrefix) not]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 00:14'!
name
	^ self nodeName qualified! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
name: aName
	self nodeName qualified: aName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 8/8/2010 14:45'!
name: aQualifiedName namespaceURI: aUri
	aQualifiedName splitQualifiedNameInto: [:prefix :localPart |
		self declarePrefix: prefix uri: aUri].
	self name: aQualifiedName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:10'!
namespaceScope
	^ namespaceScope ifNil: [namespaceScope := XMLNamespaceScope new]! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/14/2010 12:52'!
namespaceURI
	^ self resolvePrefix: self prefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:56'!
nextElement
	^ self parent ifNotNil: [self parent elementAfter: self]! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 7/27/2010 12:53'!
nodeName
	^ name ifNil: [
		(name := XMLNodeName new)
			addObserver: self;
			yourself]! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 7/11/2010 02:25'!
nodeNameChangedFrom: anOldName to: aNewName
	self validateName.
	self
		notifyOf: #renamedElement:from:to:
		with: self
		with: anOldName
		with: aNewName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 3/17/2010 03:48'!
prefix
	^ self nodeName prefix! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/15/2010 15:15'!
prefix: aPrefix
	self nodeName prefix: aPrefix! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/30/2010 19:57'!
previousElement
	^ self parent ifNotNil: [self parent elementBefore: self]! !

!XMLElement methodsFor: 'printing' stamp: 'JAAyer 11/8/2010 03:05'!
printXMLOn: aWriter
	| tagWriter |

	(tagWriter := aWriter tag)
		name: self name.

	self hasNamespaces
		ifTrue: [
			tagWriter
				xmlns: self defaultNamespace;
				xmlnsDeclarations: self namespaceScope prefixMappings].
	tagWriter
		attributes: self attributes;
		with: [super printXMLOn: aWriter]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/19/2010 10:58'!
qualifiedName
	self deprecated: 'use #name instead'.

	^ self nodeName qualified! !

!XMLElement methodsFor: 'searching' stamp: 'JAAyer 3/7/2010 04:00'!
recurse: aBlock
	"See superclass version."

	aBlock value: self.

	super recurse: aBlock.! !

!XMLElement methodsFor: 'searching' stamp: 'JAAyer 3/7/2010 04:01'!
recurseUntilTrue: aBlock
	"See superclass version."

	(aBlock value: self)
		ifTrue: [^ self].

	^ super recurseUntilTrue: aBlock.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:15'!
removeAttribute: aName
	^ (self attributeNodes removeAttributeNamed: aName) value! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:54'!
removeAttributeNode: aNode
	^ self attributeNodes remove: aNode! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 10/24/2010 18:08'!
removedAttribute: anAttribute
	anAttribute
		removeObserver: self;
		element: nil! !

!XMLElement methodsFor: 'notifications' stamp: 'JAAyer 8/6/2010 20:16'!
renamedAttribute: anAttribute from: anOldName to: aNewName
	self validateAttributeName: aNewName! !

!XMLElement methodsFor: 'namespaces' stamp: 'JAAyer 6/20/2010 23:08'!
resolvePrefix: aPrefix
	^ self namespaceScope resolvePrefix: aPrefix! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 7/12/2010 14:40'!
setAttributes: aDictionaryOrArray
	self attributeNodes removeAll.
	aDictionaryOrArray associationsDo: [:each |
		self attributeNodes
			addName: each key
			value: each value].! !

!XMLElement methodsFor: 'initialization' stamp: 'JAAyer 10/24/2010 18:19'!
setQualifiedName: aQualifiedName prefix: aPrefix localName: aLocalName attributes: anAttributeDictionary namespaceScope: aNamespaceScope
	self nodeName
		setQualified: aQualifiedName
		prefix: aPrefix
		localPart: aLocalName.

	anAttributeDictionary
		ifNotEmpty: [
			"By not observing 'attributes' until after it is filled, we avoid
			re-validating the attributes, which the parser should have already done"
			attributes := XMLAttributeList new: anAttributeDictionary size.
			anAttributeDictionary keysAndValuesDo: [:key :value |
				(attributes addName: key value: value)
					addObserver: self;
					element: self].
			attributes addObserver: self].

	namespaceScope := aNamespaceScope.! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodes
	^ self nodes select: [:each | each isStringNode]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:08'!
stringNodesDo: aBlock
	self nodes do: [:each | each isStringNode ifTrue: [aBlock value: each]]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 01:44'!
strings
	^ self stringNodes collect: [:each | each string]! !

!XMLElement methodsFor: 'enumerating' stamp: 'JAAyer 8/9/2010 01:44'!
stringsDo: aBlock
	self stringNodesDo: [:each | aBlock value: each string]! !

!XMLElement methodsFor: 'accessing' stamp: 'JAAyer 10/19/2010 10:57'!
tag
	self deprecated: 'use #name instead'.

	^ self name.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 6/20/2010 23:08'!
updateEnclosedScopesWith: aBlock where: aConditional
	"Updates the receiver's scope and the scopes of all descendants with aBlock
	where aConditional is true. It stops descending down a path of a subtree
	as soon as aConditional is false, but continues down other paths."

	(aConditional value: self namespaceScope)
		ifTrue: [aBlock value: self namespaceScope]
		ifFalse: [^ self].

	self elementsDo: [:each |
		each updateEnclosedScopesWith: aBlock where: aConditional]! !

!XMLElement methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 15:33'!
usesNamespaces
	^ namespaceScope notNil! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/6/2010 23:46'!
validateAttributeName: anAttributeName
	self usesNamespaces ifFalse: [^ self].

	self namespaceScope
		validatePrefixedAttributeName: anAttributeName
		in: self attributeNodes attributesByName.! !

!XMLElement methodsFor: 'private' stamp: 'JAAyer 8/7/2010 13:42'!
validateName
	(self usesNamespaces and: [self hasPrefix])
		ifTrue: [
			self namespaceScope
				validatePrefix: self prefix
				ofElementName: self name]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 2/25/2010 14:37'!
addElement: anElement
	self addNode: anElement! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 7/12/2010 03:20'!
addedElement: anElement
	anElement document: self document! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodes
	^ self nodes select: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 7/22/2010 08:09'!
contentNodesDo: aBlock
	self nodes do: [:each | each isContentNode ifTrue: [aBlock value: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
contentStringAt: aName
	^ (self elementAt: aName ifAbsent: [^ '']) contentString! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:19'!
document
	self subclassResponsibility! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 11:45'!
elementAfter: anElement
	^ self elements after: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
elementAt: aName
	^ self nodes elementNamed: aName! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
elementAt: aName ifAbsent: aBlock
	^ self nodes elementNamed: aName ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 08:56'!
elementAt: aName namespaceURI: aUri
	^ self nodes elementNamed: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/28/2010 08:57'!
elementAt: aName namespaceURI: aUri ifAbsent: aBlock
	^ self nodes elementNamed: aName namespaceURI: aUri ifAbsent: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
elementAtAny: aNameArray
	^ self elementAtAny: aNameArray ifAbsent: [nil]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
elementAtAny: aNameArray ifAbsent: aBlock
	| answer |

	aNameArray do: [:each |
		(answer := self elementAt: each)
			ifNotNil: [^ answer]].
	^ aBlock value.! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 11:45'!
elementBefore: anElement
	^ self elements before: anElement! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 5/4/2010 13:18'!
elements
	^ self nodes elements! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 10/24/2010 16:14'!
elementsAt: aName
	^ self nodes elementsNamed: aName! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 10/24/2010 16:14'!
elementsAt: aName do: aBlock
	(self nodes elementsNamed: aName) do: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/27/2010 08:52'!
elementsAt: aName namespaceURI: aUri
	^ self nodes elementsNamed: aName namespaceURI: aUri! !

!XMLNodeWithElements methodsFor: 'enumerating' stamp: 'JAAyer 6/26/2010 00:55'!
elementsDo: aBlock
	self nodes elementsDo: aBlock! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 10:34'!
firstElement
	^ self elements first! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
firstTagNamed: aName 
	"Return the first encountered node with the specified tag. Pass the message on"

	^ self recurseUntilTrue: [:each | each isNamed: aName]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
firstTagNamed: aName with: aBlock
	"Return the first encountered node with the specified tag that
	allows the block to evaluate to true. Pass the message on"

	^ self recurseUntilTrue: [:each |
		(each isNamed: aName)
			and: [aBlock valueWithPossibleArgument: each]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
firstTagNamedAny: aNameArray
	"Return the first encountered node with any of the specified tag names. Pass the message on"

	^ self recurseUntilTrue: [:each | each isNamedAny: aNameArray]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
firstTagNamedAny: aNameArray with: aBlock
	"Return the first encountered node with any of the specified tag names that
	allows the block to evaluate to true. Pass the message on"

	^ self recurseUntilTrue: [:each |
		(each isNamedAny: aNameArray)
			and: [aBlock valueWithPossibleArgument: each]]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 10/19/2010 11:13'!
hasContentNodes
	^ self nodes anySatisfy: [:each | each isContentNode]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/28/2010 09:54'!
hasElements
	"Faster version for #recurse*: methods to avoid initializing the node list in leaf elements"
	^ nodes notNil and: [nodes hasElements]! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 16:14'!
includesElement: aName
	^ self nodes includesElementNamed: aName! !

!XMLNodeWithElements methodsFor: 'parsing' stamp: 'JAAyer 11/13/2010 09:00'!
innerXML: aString
	| parsedDocument newNodes |

	parsedDocument := XMLDOMParser
		parse: aString
		usingNamespaces: self usesNamespaces.
	newNodes := parsedDocument nodes copy.
	parsedDocument removeNodes.
	self
		removeNodes;
		addNodes: newNodes.! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:15'!
isContentNode
	^ true! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 3/12/2010 10:34'!
lastElement
	^ self elements last! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 8/7/2010 20:11'!
namespaceScope
	^ XMLNamespaceScope new! !

!XMLNodeWithElements methodsFor: 'private' stamp: 'JAAyer 7/20/2010 12:31'!
nodesClass
	^ XMLNodeList! !

!XMLNodeWithElements methodsFor: 'printing' stamp: 'JAAyer 5/5/2010 15:38'!
printContentOn: aStream
	self contentNodesDo: [:each | each printContentOn: aStream]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 7/20/2010 10:57'!
recurse: aBlock
	"Descend depth-first visiting each element with aBlock."

	self hasElements
		ifTrue: [self elementsDo: [:each | each recurse: aBlock]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 7/20/2010 10:57'!
recurseUntilTrue: aBlock
	"Descend depth-first visiting each element with aBlock until one such evaluation
	of aBlock with an element is true, then return that element. If no evaluation is
	true, then return nil."
	| result |

	self hasElements
		ifTrue: [
			self elementsDo: [:each |
				(result := each recurseUntilTrue: aBlock)
					ifNotNil: [^ result]]].
	^ nil.! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 7/21/2010 12:01'!
removedElement: anElement
	anElement document: nil! !

!XMLNodeWithElements methodsFor: 'notifications' stamp: 'JAAyer 6/22/2010 22:41'!
renamedElement: anElement from: anOldName to: aNewName! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
tagsNamed: aName childrenDo: aOneArgumentBlock
	"Evaluate aOneArgumentBlock for all children who match"

	self deprecated: 'use #elementsAt:do: instead'.

	self elementsAt: aName do: aOneArgumentBlock.! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
tagsNamed: aName childrenDoAndRecurse: aOneArgumentBlock
	"Evaluate aOneArgumentBlock for all children who match and recurse"

	self deprecated: 'use "node elementsDo: [:each | each tagsNamed: aName do: [...]]" instead'.

	self elementsDo: [:each | each tagsNamed: aName do: aOneArgumentBlock]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
tagsNamed: aName contentsDo: aBlock
	"Evaluate aBlock for all of the contents of the receiver.
	The receiver has no tag, so pass the message on"

	self deprecated: 'use "node tagsNamed: aName do: [:each | each stringNodes...]" instead'.

	self tagsNamed: aName do: [:each | each stringNodesDo: aBlock]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
tagsNamed: aName do: aOneArgumentBlock
	"Search for nodes with tag aName. When encountered evaluate aOneArgumentBlock"

	self recurse: [:each |
		(each isNamed: aName)
			ifTrue: [aOneArgumentBlock valueWithPossibleArgument: each]]! !

!XMLNodeWithElements methodsFor: 'searching' stamp: 'JAAyer 10/24/2010 16:14'!
tagsNamedAny: aNameCollection do: aOneArgumentBlock
	"Search for nodes with tag aName. When encountered evaluate aOneArgumentBlock"

	self recurse: [:each |
		(each isNamedAny: aNameCollection)
			ifTrue: [aOneArgumentBlock valueWithPossibleArgument: each]]! !

!XMLNodeWithElements methodsFor: 'accessing' stamp: 'JAAyer 7/12/2010 03:23'!
topElement
	self deprecated: 'use #firstElement instead'.

	^ self firstElement.! !

!XMLNodeWithElements methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 15:34'!
usesNamespaces
	^ true! !

XMLNode subclass: #XMLPI
	instanceVariableNames: 'target data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLPI commentStamp: 'JAAyer 6/25/2010 14:06' prior: 0!
This class represents a processing instruction.
!

!XMLPI class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
target: targetName data: aString
	^ self new
		target: targetName;
		data: aString! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 03:02'!
data
	^ data ifNil: [data := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
data: aString
	data := aString! !

!XMLPI methodsFor: 'testing' stamp: 'JAAyer 4/13/2010 21:43'!
isPI
	^ true! !

!XMLPI methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter pi
		target: self target;
		data: self data! !

!XMLPI methodsFor: 'accessing' stamp: 'JAAyer 7/21/2010 12:42'!
target
	^ target ifNil: [target := '']! !

!XMLPI methodsFor: 'accessing' stamp: 'mir 1/17/2002 13:02'!
target: aString
	target := aString! !

XMLNode subclass: #XMLString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLString commentStamp: '<historical>' prior: 0!
This class represents string nodes, or nodes that contain no markup. XMLString objects can be created directly using the class-side #string: message or by sending a String object #asXMLString. When printing, they XML-escape their content.!

XMLString subclass: #XMLCData
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLCData commentStamp: '<historical>' prior: 0!
This class represents a preserved CDATA section in the document. It behaves like a string, but its printed form differs.!

!XMLCData methodsFor: 'testing' stamp: 'JAAyer 8/11/2010 22:16'!
isCData
	^ true! !

!XMLCData methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter cdata: self string! !

!XMLString class methodsFor: 'instance creation' stamp: 'JAAyer 2/25/2010 01:48'!
string: aString
	^ self new string: aString! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 6/10/2010 04:54'!
addContent: aStringOrStringNode
	self string: self string, (self toString: aStringOrStringNode)! !

!XMLString methodsFor: 'converting' stamp: 'JAAyer 7/22/2010 05:19'!
asXMLString
	^ self! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 7/20/2010 10:26'!
isContentNode
	^ true! !

!XMLString methodsFor: 'testing' stamp: 'JAAyer 4/12/2010 22:35'!
isStringNode
	^ true! !

!XMLString methodsFor: 'printing' stamp: 'JAAyer 9/2/2010 13:25'!
printContentOn: aStream
	| lastChar |

	self string ifEmpty: [^ self].

	((lastChar := aStream peekLast) isNil
		or: [lastChar isSeparator
			or: [self string first isSeparator]])
		ifFalse: [aStream space].

	aStream nextPutAll: self string.! !

!XMLString methodsFor: 'printing' stamp: 'JAAyer 11/5/2010 18:50'!
printXMLOn: aWriter
	aWriter string: self string! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 7/22/2010 05:19'!
string
	^ string ifNil: [string := '']! !

!XMLString methodsFor: 'accessing' stamp: 'JAAyer 2/23/2010 02:55'!
string: aString
	string := self toString: aString! !

!XMLString methodsFor: 'private' stamp: 'JAAyer 10/28/2010 15:22'!
toString: aStringOrStringNode
	^ (aStringOrStringNode isKindOf: XMLString)
		ifTrue: [aStringOrStringNode string]
		ifFalse: [aStringOrStringNode]! !

Object subclass: #XMLNodeFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeFactory commentStamp: '<historical>' prior: 0!
This class is the default node factory. Its accessor messages return the class objects used by the DOM parser to build a node tree. You can override these messages to return different class objects and inject an instance of your factory using #nodeFactory:.!

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
cdataClass
	^ XMLCData! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:20'!
classForElement: aQualifiedName prefix: aPrefix uri: aNamespaceUri localName: aLocalName
	^ self elementClass! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
commentClass
	^ XMLComment! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
doctypeDeclarationClass
	^ XMLDoctypeDeclaration! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:19'!
documentClass
	^ XMLDocument! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
elementClass
	^ XMLElement! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
piClass
	^ XMLPI! !

!XMLNodeFactory methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 03:18'!
stringClass
	^ XMLString! !

XMLNodeFactory subclass: #XMLPluggableElementFactory
	instanceVariableNames: 'elementClass elementClassesByName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLPluggableElementFactory commentStamp: '<historical>' prior: 0!
This class is a pluggable factory that can map elements to different XMLElement subclasses based on the name and namespace information of those elements. You have to create an instance, configure it to handle certain elements with certain classes, and then inject your instance into a DOM parser using #nodeFactory: before parsing. Here is an example of its use:

	doc := (XMLDOMParser on: someXML)
		nodeFactory:
			(XMLPluggableElementFactory new
				elementClass: GenericElement;
				handleElement: 'user' withClass: UserElement;
				handleElement: 'report' withClass: ReportElement;
				handleElement: 'report' namespaceURI: 'http://specialreprot' withClass: SpecialReportElement)
		parseDocument.!

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/14/2010 21:57'!
classForElement: aName prefix: aPrefix uri: aUri localName: aLocalName
	^ (self classForElement: aName uri: aUri)
		ifNil: [
			aPrefix
				ifEmpty: [self elementClass]
				ifNotEmpty: [
					(self classForElement: aLocalName uri: aUri)
						ifNil: [self elementClass]]]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/14/2010 21:55'!
classForElement: aName uri: aUri
	| classesForElement |

	classesForElement :=
		self elementClassesByName
			at: aName
			ifAbsent: [^ nil].

	^ classesForElement
		at: aUri
		ifAbsent: [
			classesForElement
				at: ''
				ifAbsent: [nil]].! !

!XMLPluggableElementFactory methodsFor: 'accessing' stamp: 'JAAyer 11/15/2010 05:05'!
elementClass
	^ elementClass ifNil: [elementClass := super elementClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/15/2010 05:04'!
elementClass: anElementClass
	elementClass := anElementClass! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/13/2010 08:36'!
elementClassesByName
	^ elementClassesByName ifNil: [elementClassesByName := Dictionary new]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName namespaceURI: aUri withClass: aClass
	(self elementClassesByName
		at: anElementName
		ifAbsentPut: [Dictionary new])
			at: aUri put: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/13/2010 19:37'!
handleElement: anElementName withClass: aClass
	self
		handleElement: anElementName
		namespaceURI: ''
		withClass: aClass! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:39'!
handleElements: anElementNameCollection namespaceURI: aNamespaceURI withClass: aClass
	anElementNameCollection do: [:each |
		self
			handleElement: each
			namespaceURI: aNamespaceURI
			withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:36'!
handleElements: anElementNameCollection withClass: aClass
	anElementNameCollection do: [:each | self handleElement: each withClass: aClass]! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:30'!
handleElementsMatchingClass: aClass
	| className |

	(className := self toNameWithoutPrefixAndElementSuffix: aClass name asString)
		ifEmpty: [^ self].

	self
		handleElement: (self toCamelCasedName: className)
		withClass: aClass.
	self
		handleElement: (self toHyphenatedName: className)
		withClass: aClass.! !

!XMLPluggableElementFactory methodsFor: 'element handling' stamp: 'JAAyer 11/20/2010 11:26'!
handleElementsMatchingClasses: aClassCollection
	aClassCollection do: [:each | self handleElementsMatchingClass: each]! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:29'!
toCamelCasedName: aString
	^ aString allButFirst copyWithFirst: aString first asLowercase! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:34'!
toHyphenatedName: aString
	| nameStream hyphenatedStream previousChar nextChar |

	nameStream := aString readStream.
	hyphenatedStream := (String new: nameStream size) writeStream.
	[nameStream atEnd]
		whileFalse: [
			((nextChar := nameStream next) isUppercase
				and: [previousChar notNil and: [previousChar isLowercase]])
				ifTrue: [hyphenatedStream nextPut: $-].
			hyphenatedStream nextPut: nextChar asLowercase.
			previousChar := nextChar].
	^ hyphenatedStream contents.! !

!XMLPluggableElementFactory methodsFor: 'private' stamp: 'JAAyer 11/20/2010 11:28'!
toNameWithoutPrefixAndElementSuffix: aString
	| nameStream |

	nameStream := aString readStream.
	[nameStream atEnd not and: [nameStream peek isUppercase]]
		whileTrue: [nameStream next].
	nameStream atEnd
		ifTrue: [^ ''].
	nameStream back.

	^ nameStream upToEnd copyReplaceAll: 'Element' with: ''.! !

Object subclass: #XMLNodeName
	uses: TXMLObservable
	instanceVariableNames: 'qualified prefix localPart notifier'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNodeName commentStamp: '<historical>' prior: 0!
The class represents a name for an element or attribute. It stores the prefix, local part and qualified name. It also notifies any observers whenever any part of the name it models changes.!

XMLNodeName class
	uses: TXMLObservable classTrait
	instanceVariableNames: ''!

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 11/1/2010 04:36'!
changeWith: aBlock
	| oldName newName |

	oldName := self qualified.
	aBlock value.
	newName := self qualified.
	newName = oldName
		ifFalse: [self notifyOf: #nodeNameChangedFrom:to: with: oldName with: newName].! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 15:01'!
expandWith: aUri
	^ aUri
		ifEmpty: [self localPart]
		ifNotEmpty: ['{', aUri, '}', self localPart]! !

!XMLNodeName methodsFor: 'testing' stamp: 'JAAyer 8/9/2010 14:46'!
isPrefixed
	^ self prefix notEmpty! !

!XMLNodeName methodsFor: 'testing' stamp: 'JAAyer 10/24/2010 18:00'!
isSameAs: aNameString
	^ self qualified = aNameString or: [self localPart = aNameString]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
localPart
	^ localPart ifNil: [localPart := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 11/21/2010 04:57'!
localPart: aLocalPart
	self changeWith: [self setPrefix: self prefix localPart: aLocalPart]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/15/2010 15:18'!
notifier
	^ notifier ifNil: [notifier := XMLNotifier new]! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 8/9/2010 14:43'!
prefix
	^ prefix ifNil: [prefix := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:17'!
prefix: aPrefix
	self changeWith: [self setPrefix: aPrefix localPart: self localPart]! !

!XMLNodeName methodsFor: 'printing' stamp: 'JAAyer 6/27/2010 15:15'!
printOn: aStream
	aStream nextPutAll: self qualified! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 7/26/2010 14:23'!
qualified
	^ qualified ifNil: [qualified := '']! !

!XMLNodeName methodsFor: 'accessing' stamp: 'JAAyer 6/27/2010 15:19'!
qualified: aQualifiedName
	aQualifiedName splitQualifiedNameInto: [:prefixPart :localName |
		self changeWith: [self setPrefix: prefixPart localPart: localName]]! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 8/14/2010 20:50'!
setPrefix: aPrefix localPart: aLocalPart
	aPrefix isEmptyOrNil
		ifTrue: [
			self
				setQualified: aLocalPart
				prefix: ''
				localPart: aLocalPart]
		ifFalse: [
			self
				setQualified: aPrefix, ':', aLocalPart
				prefix: aPrefix
				localPart: aLocalPart]! !

!XMLNodeName methodsFor: 'private' stamp: 'JAAyer 6/27/2010 15:20'!
setQualified: aQualifiedName prefix: aPrefix localPart: aLocalPart
	qualified := aQualifiedName.
	prefix := aPrefix.
	localPart := aLocalPart.! !

Object subclass: #XMLNotifier
	instanceVariableNames: 'observers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser-Nodes'!

!XMLNotifier commentStamp: '<historical>' prior: 0!
This class is used by objects using the TXMLObservable Trait to send notifications to objects that register themselves as observers. The notifications are selectors that observers are expected to implement, typically under the category "notifications."

It stores observers in an Array grown and shrunk with #copyWith: and #copyWithout: rather than an OrderedCollection for better memory usage and faster performance (because each node only has at most two observers).!

!XMLNotifier methodsFor: 'observing' stamp: 'JAAyer 7/20/2010 10:05'!
addObserver: anObject
	observers :=
		observers
			ifNil: [Array with: anObject]
			ifNotNil: [observers copyWith: anObject]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: anArgument
	observers ifNotNil: [
		observers do: [:each |
			each perform: aSelector with: anArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument
	observers ifNotNil: [
		observers do: [:each |
			each
				perform: aSelector
				with: aFirstArgument
				with: aSecondArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:04'!
notifyOf: aSelector with: aFirstArgument with: aSecondArgument with: aThirdArgument
	observers ifNotNil: [
		observers do: [:each |
			each
				perform: aSelector
				with: aFirstArgument
				with: aSecondArgument
				with: aThirdArgument]]! !

!XMLNotifier methodsFor: 'notification' stamp: 'JAAyer 7/20/2010 10:05'!
notifyOf: aSelector withArguments: anArray
	observers ifNotNil: [
		observers do: [:each |
			each perform: aSelector withArguments: anArray]]! !

!XMLNotifier methodsFor: 'accessing' stamp: 'JAAyer 7/20/2010 10:02'!
observers
	^ observers ifNil: [#()]! !

!XMLNotifier methodsFor: 'observing' stamp: 'JAAyer 7/20/2010 10:03'!
removeObserver: anObject
	(observers := observers copyWithout: anObject)
		ifEmpty: [observers := nil]! !

Object subclass: #XMLTokenizer
	instanceVariableNames: 'streamReader streamWriter entities externalEntities parameterEntities isValidating parsingMarkup'
	classVariableNames: 'CharEscapes LiteralChars NameDelimiters'
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLTokenizer commentStamp: '<historical>' prior: 0!
This class reads XML tokens from a stream using a nested stream reader. To create a parser, subclass this class and override the #handle- messages to handle tokens as they are read. To cause a token to be read, send #nextToken to an instance.!

XMLTokenizer subclass: #SAXDriver
	instanceVariableNames: 'saxHandler useNamespaces validateAttributes languageEnvironment scopeManager wellFormednessChecker'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!SAXDriver commentStamp: 'JAAyer 3/23/2010 16:10' prior: 0!
SAXDriver together with SAXHandler provides a SAX-like API for XML parsing. A SAXDriver processes XML tokens and signals SAX events to a SAXHandler. To use it, create a subclass of SAXHandler and override messages that handle SAX events of interest. See "instance creation" on the class-side of SAXHandler for ways to instantiate such a parser.!

!SAXDriver methodsFor: 'content' stamp: 'JAAyer 3/5/2010 10:38'!
activeSaxHandler
	saxHandler eod
		ifTrue: [self errorExpected: 'No more data expected.'].
	^ saxHandler.! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 12:52'!
currentScope
	^ self usesNamespaces
		ifTrue: [self scopeManager currentScope]
		ifFalse: [nil]! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 10/15/2010 14:21'!
declareNamespaces: aDictionary
	aDictionary keysAndValuesDo: [:prefix :uri |
		self scopeManager declarePrefix: prefix uri: uri.
		self activeSaxHandler startPrefixMapping: prefix uri: uri]! !

!SAXDriver methodsFor: 'errors' stamp: 'JAAyer 3/22/2010 21:25'!
errorBadPrefix: aPrefix inStartTag: aStartTag
	XMLNamespaceException signal:
		'Start tag <', aStartTag, '> refers to unmapped prefix ', aPrefix! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/5/2010 10:33'!
handleCData: aString
	self activeSaxHandler
		startCData;
		characters: aString;
		endCData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/5/2010 10:33'!
handleComment: aString
	self activeSaxHandler 
		comment: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 10/15/2010 15:08'!
handleEndDocument
	self wellFormednessChecker ensureNoElementsRemainOpen.
	self saxHandler endDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 10/15/2010 15:10'!
handleEndTag: anElementName
	| prefix localName namespaceURI |

	self wellFormednessChecker endedElement: anElementName.

	anElementName splitQualifiedNameInto: [:prefixPart :localPart |
		prefix := prefixPart.
		localName := localPart].
	self usesNamespaces
		ifTrue: [namespaceURI := self scopeManager resolvePrefix: prefix].

	self activeSaxHandler 
		endElement: anElementName
		prefix: prefix
		uri: namespaceURI
		localName: localName.

	self usesNamespaces
		ifTrue: [
			self undeclareNamespaces.
			self scopeManager exitScope].! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/5/2010 10:33'!
handlePCData: aString
	self languageEnvironment
		ifNotNil: [aString applyLanguageInfomation: self languageEnvironment].
	self activeSaxHandler 
		characters: aString! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/5/2010 10:33'!
handlePI: piTarget data: piData
	self activeSaxHandler 
		processingInstruction: piTarget data: piData! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:29'!
handleStartDocument
	self saxHandler startDocument! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 11/14/2010 21:43'!
handleStartTag: anElementName attributes: anAttributeDictionary namespaces: aNamespaceDictionary
	| prefix localName namespaceURI |

	self wellFormednessChecker startedElement: anElementName.
	(anAttributeDictionary includesKey: 'xml:lang')
		ifTrue: [self languageEnvironment: (anAttributeDictionary at: 'xml:lang')].
	anElementName splitQualifiedNameInto: [:prefixPart :localPart |
		prefix := prefixPart.
		localName := localPart.
		namespaceURI := ''].

	self usesNamespaces ifTrue: [
		self scopeManager enterScope.
		self declareNamespaces: aNamespaceDictionary.
		namespaceURI := self scopeManager resolvePrefix: prefix.
		self
			validateStartTag: anElementName
			prefix: prefix
			attributes: anAttributeDictionary].

	self activeSaxHandler 
		startElement: anElementName
		prefix: prefix
		uri: namespaceURI
		localName: localName
		attributes: anAttributeDictionary! !

!SAXDriver methodsFor: 'handling tokens' stamp: 'JAAyer 3/5/2010 10:33'!
handleWhitespace: aString
	self activeSaxHandler 
		ignorableWhitespace: aString! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 07:21'!
languageEnvironment
	^ languageEnvironment! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 3/2/2010 03:31'!
languageEnvironment: anIsoString
	languageEnvironment := LanguageEnvironment localeID: (LocaleID isoString: anIsoString)! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 3/14/2010 10:52'!
saxHandler
	^ saxHandler! !

!SAXDriver methodsFor: 'accessing' stamp: 'mir 8/11/2000 17:52'!
saxHandler: aHandler
	saxHandler := aHandler! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 10/15/2010 12:52'!
scopeManager
	^ scopeManager ifNil: [scopeManager := XMLNamespaceScopeManager new]! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/5/2010 20:43'!
undeclareNamespaces
	| currentScope enclosingScope |

	currentScope := self scopeManager currentScope.
	currentScope hasNamespaces
		ifFalse: [^ self].
	enclosingScope := self scopeManager enclosingScope.

	currentScope prefixesAndURIsDo: [:prefix :uri |
		(enclosingScope includesPrefix: prefix mappedTo: uri)
			ifFalse: [self activeSaxHandler endPrefixMapping: prefix]].

	(currentScope defaultNamespace notNil
		and: [currentScope defaultNamespace ~= enclosingScope defaultNamespace])
		ifTrue: [self activeSaxHandler endPrefixMapping: '']! !

!SAXDriver methodsFor: 'testing' stamp: 'JAAyer 11/10/2010 06:00'!
usesNamespaces
	^ useNamespaces ifNil: [useNamespaces := true]! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 11/10/2010 05:33'!
usesNamespaces: aBoolean
	useNamespaces := aBoolean! !

!SAXDriver methodsFor: 'namespaces' stamp: 'JAAyer 11/6/2010 02:03'!
validateStartTag: aStartTag prefix: aPrefix attributes: anAttributeDictionary
	aPrefix ifNotEmpty: [
		self scopeManager
			validatePrefix: aPrefix
			ofElementName: aStartTag].

	self scopeManager validatePrefixedAttributeNames: anAttributeDictionary.! !

!SAXDriver methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:17'!
wellFormednessChecker
	^ wellFormednessChecker ifNil: [wellFormednessChecker := XMLWellFormednessChecker new]! !

!XMLTokenizer class methodsFor: 'class initialization' stamp: 'JAAyer 3/11/2010 08:38'!
initialize
	"XMLTokenizer initialize"

	CharEscapes := CharacterSet newFrom: #( $& $" $' $> $< ).

	LiteralChars := CharacterSet newFrom: #( $: $- $: $= $.).
	0 to: 255 do: [:i | 
		| char |
		char := i asCharacter.
		(char isDigit or: [char isLetter])
		ifTrue: [LiteralChars add: char]].

	NameDelimiters := CharacterSet new.
	#(9 10 12 13 32 61 "$= asInteger 61" 62 "$> asInteger" 47 "$/ asInteger")
		do: [:each | NameDelimiters add: each asCharacter].! !

!XMLTokenizer class methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 07:53'!
isCharEscape: entityValue
	^entityValue size = 1
		and: [CharEscapes includes: entityValue first]! !

!XMLTokenizer class methodsFor: 'instance creation' stamp: 'JAAyer 11/10/2010 05:23'!
on: aStringOrStream
	^self new stream:
		(aStringOrStream isStream
			ifTrue: [aStringOrStream]
			ifFalse: [aStringOrStream readStream])! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/14/2010 12:03'!
checkAndExpandReference: parsingContext
	| referenceString nextChar |
	nextChar := streamReader peek.
	self isValidating
		ifFalse: [^ nil].

	nextChar == $&
		ifTrue: [
			streamReader next.
			streamReader peek == $#
				ifTrue: [^ streamReader pushStream: (ReadStream on: self nextCharReference asString)].
			referenceString := self nextLiteral.
			self expectNext: $;.
			self handleEntity: referenceString in: parsingContext ]
		ifFalse: [
			((nextChar == $%
				and: [self parsingMarkup])
				and: [parsingContext == #entityValue])
				ifTrue: [
					streamReader skipSeparators.
					referenceString := self nextLiteral.
					self handleEntity: referenceString in: parsingContext]].

	self expectMore.
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:46'!
conditionalInclude: aKeyword
	aKeyword = 'INCLUDE'
		ifTrue: [^ true].
	aKeyword = 'IGNORE'
		ifTrue: [^ false].
	^ self conditionalInclude: (self parameterEntity: aKeyword) value! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 11/13/2000 18:19'!
endParsingMarkup
	parsingMarkup := false! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/16/2010 16:39'!
entities
	^ entities ifNil: [entities := self initEntities]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/16/2010 12:45'!
entity: aReference
	self isValidating
		ifFalse: [^ DTDEntityDeclaration name: aReference value: ''].

	^ self entities
		at: aReference
		ifAbsentPut: [self parseError: 'Undefined entity ', aReference printString]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 09:47'!
entity: aReference put: aValue
	"Only the first declaration of an entity is valid so if there is already
	one don't register the new value."
	self entities at: aReference ifAbsentPut: [aValue]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'JAAyer 3/14/2010 12:02'!
errorExpected: expectedString
	self parseError: 'Expected ', expectedString! !

!XMLTokenizer methodsFor: 'errors' stamp: 'JAAyer 3/16/2010 12:49'!
errorExpected: anExpectedCharacterOrString butGot: aReceivedCharacterOrString
	| expectedString receivedString |

	expectedString := anExpectedCharacterOrString asString.	
	(receivedString := (aReceivedCharacterOrString ifNil: ['']) asString)
			ifEmpty: [receivedString := 'nothing'].

	self errorExpected: expectedString, ' but got ', receivedString.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 12:49'!
expectLiteral: anExpectedLiteral
	| nextLiteral |

	((nextLiteral := self nextLiteral) = anExpectedLiteral)
		ifFalse: [self errorExpected: anExpectedLiteral butGot: nextLiteral].
	^ nextLiteral.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 16:40'!
expectMore
	streamReader peek ifNil: [self errorExpected: 'more characters']! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 12:49'!
expectNext: aCharacter
	| nextChar |

	(nextChar := streamReader next) == aCharacter
		ifFalse: [
			self
				errorExpected: aCharacter
				butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 12:46'!
expectQuote
	| nextChar expected |

	(((nextChar := streamReader next) == $")
		or: [nextChar == $'])
			ifFalse: [
				self
					errorExpected: 'quote character delimiter'
					butGot: nextChar].
	^ nextChar.! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/19/2010 14:19'!
externalEntities
	^ externalEntities ifNil: [externalEntities := Dictionary new]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 09:48'!
externalEntity: aReference
	^ self entities at: aReference ifAbsentPut: ['']! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 11/13/2000 16:04'!
handleCData: aString
	self log: 'CData: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:37'!
handleComment: aString
	self log: 'Comment: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:27'!
handleEndDocument
	self log: 'End Doc '! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:38'!
handleEndTag: aString
	self log: 'End tag: ' , aString! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 08:28'!
handleEntity: referenceString in: parsingContext 

	| entity entityValue |
	entity := self entity: referenceString.
	entityValue := entity valueForContext: parsingContext.
	(self class isCharEscape: entityValue)
		ifTrue: [entityValue := entity reference].
	streamReader pushStream: (ReadStream on: entityValue asString)! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:38'!
handlePCData: aString
	self log: 'PCData: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 12/11/2000 16:10'!
handlePI: piTarget data: piData
	self log: 'PI: ' , piTarget , ' data ' , piData! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 18:26'!
handleStartDocument
	self log: 'Start Doc'! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'mir 8/14/2000 11:39'!
handleStartTag: tagName attributes: attributes
	self log: 'Start tag: ' , tagName.
	attributes keysAndValuesDo: [:key :value |
		self log: key , '->' , value]! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'JAAyer 3/5/2010 04:11'!
handleStartTag: tagName attributes: attributes namespaces: namespaces
	self handleStartTag: tagName attributes: attributes! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'cwp 6/17/2003 21:08'!
handleWhitespace: aString
	self log: 'Whitespace: ' , aString! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'JAAyer 3/2/2010 22:54'!
handleXMLDecl: attributes
	attributes keysAndValuesDo: [:key :value |
		self log: key , '->' , value]! !

!XMLTokenizer methodsFor: 'handling tokens' stamp: 'JAAyer 11/14/2010 21:31'!
handleXMLVersion: aVersion encoding: anEncoding standalone: aYesOrNo
	| versionNumber standaloneBoolean |

	aVersion
		ifEmpty: [versionNumber := nil]
		ifNotEmpty: [versionNumber := aVersion asNumber].

	(aYesOrNo = 'yes' or: [aYesOrNo = 'no'])
		ifTrue: [standaloneBoolean := aYesOrNo = 'yes']
		ifFalse: [standaloneBoolean := nil].

	self saxHandler
		xmlVersion: versionNumber
		encoding: anEncoding
		standalone: standaloneBoolean! !

!XMLTokenizer methodsFor: 'entities' stamp: 'mir 11/23/2007 15:45'!
initEntities
	| ents |
	ents := Dictionary new.
	ents
		at: 'amp' put: (DTDEntityDeclaration name: 'amp' value: '&');
		at: 'quot' put: (DTDEntityDeclaration name: 'quot' value: '"');
		at: 'apos' put: (DTDEntityDeclaration name: 'apos' value: '''');
		at: 'gt' put: (DTDEntityDeclaration name: 'gt' value: '>');
		at: 'lt' put: (DTDEntityDeclaration name: 'lt' value: '<').
	^ents! !

!XMLTokenizer methodsFor: 'initialization' stamp: 'JAAyer 11/10/2010 05:27'!
initialize
	streamReader := XMLNestedStreamReader new.
	streamWriter := XMLNestedStreamWriter new.
	parsingMarkup := false.
	isValidating := true.! !

!XMLTokenizer methodsFor: 'testing' stamp: 'JAAyer 3/11/2010 06:44'!
isValidating
	^ isValidating! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 06:45'!
isValidating: aBoolean
	isValidating := aBoolean! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 12/7/2000 16:46'!
log: aString
	"Transcript show: aString; cr"! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/5/2010 20:45'!
nextAttributeInto: attributes namespaces: namespaces

	| attrName attrValue |
	attrName := self nextName.
	streamReader skipSeparators.
	self expectNext: $=.
	streamReader skipSeparators.
	attrValue := self nextAttributeValue.

	(self usesNamespaces
		and: [attrName beginsWith: 'xmlns'])
		ifTrue: [attrName size > 6
			ifTrue: [namespaces at: (attrName copyFrom: 7 to: attrName size) put: attrValue]
			ifFalse: [namespaces at: '' put: attrValue]]
		ifFalse: [attributes at: attrName put: attrValue].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/23/2010 18:09'!
nextAttributeValue
	| delimiterChar |

	delimiterChar := self expectQuote.
	^ streamWriter writeWith: [:writeStream |
		self nextPCDataDelimitedBy: delimiterChar putOn: writeStream.
		self expectNext: delimiterChar.
		writeStream stringContents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 09:12'!
nextCDataContent
	| cdata |

	cdata := streamReader upToAll: ']]>'.
	self handleCData: cdata
! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/15/2010 04:36'!
nextCDataOrConditional

	| nextChar conditionalKeyword |
	"Skip ["
	streamReader next.
	streamReader skipSeparators.
	nextChar := streamReader peek.
	nextChar == $%
		ifTrue: [
			self checkAndExpandReference: (self parsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
			conditionalKeyword := self nextLiteral.
			streamReader skipSeparators.
			self expectNext: $[.
			streamReader skipSeparators.
			^ self nextIncludeSection: (self conditionalInclude: conditionalKeyword)].

	self
		expectLiteral: 'CDATA';
		expectNext: $[;
		nextCDataContent.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 10/15/2010 14:47'!
nextCharReference
	| base charValue |
	streamReader next == $#
		ifFalse: [self errorExpected: 'character reference'].
	streamReader peek == $x
		ifTrue: [
			streamReader next.
			base := 16]
		ifFalse: [base := 10].

	(charValue := streamReader nextIntegerWithBase: base)
		ifNil: [self errorExpected: 'number'].
	self expectNext: $;.
	^ Unicode value: charValue.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 14:37'!
nextComment
	"Skip first -"
	streamReader next.
	self expectNext: $-.
	self handleComment: (streamReader upToAll: '-->')! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/15/2010 04:32'!
nextDocType
	self expectLiteral: 'DOCTYPE'.
	self startParsingMarkup.
	^ self nextDocTypeDecl.! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/19/2010 18:33'!
nextDocTypeDecl
	| nextChar |
	streamReader skipSeparators.
	self nextLiteral.
	streamReader skipSeparators.
	streamReader peek == $[
		ifFalse: [[nextChar := streamReader peek.
				nextChar == $> or: [nextChar == $[ ]] whileFalse: [streamReader next]].
	streamReader peek == $[
		ifTrue: [
			streamReader next.
			[streamReader skipSeparators.
			streamReader peek == $]] whileFalse: [
				self checkAndExpandReference: #dtd.
				self nextMarkupToken].
			self expectNext: $]].
	streamReader skipSeparators.
	self expectNext: $>.

	self endParsingMarkup! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 09:08'!
nextEndTag
	| tagName |
	"Skip /"
	streamReader next.
	tagName := self nextName.
	streamReader skipSeparators.
	self expectNext: $>.
	self handleEndTag: tagName! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/11/2010 08:25'!
nextEntityDeclaration
	| entityName entityDef referenceClass reference |
	streamReader skipSeparators.
	referenceClass := streamReader peek == $%
		ifTrue: [
			streamReader next.
			streamReader skipSeparators.
			DTDParameterEntityDeclaration]
		ifFalse: [DTDEntityDeclaration].
	entityName := self nextLiteral.
	streamReader skipSeparators.
	entityDef := (streamReader peek == $" or: [streamReader peek == $'])
		ifTrue: [self nextEntityValue]
		ifFalse: [self nextExternalId].
	streamReader skipUpTo: $>.
	reference := referenceClass name: entityName value: entityDef.
	reference registerIn: self.
	^reference! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/19/2010 14:26'!
nextEntityValue
	| delimiterChar entityValueStream nextChar nextPeek referenceString entity entityValue |
	delimiterChar := self expectQuote.
	entityValueStream := WriteStream on: (String new: 32).
	[
	self expectMore.
	nextPeek := nextChar := streamReader peek.
	nextChar == $&
		ifTrue: [
			streamReader next.
			streamReader peek == $#
				ifTrue: [
					nextPeek := nil.
					nextChar := self nextCharReference]
				ifFalse: [
					referenceString := self nextLiteral.
					self expectNext: $;.
					entity := self entity: referenceString.
					entityValue := entity valueForContext: #entityValue.
					streamReader pushStream: (ReadStream on: entityValue asString).
					nextPeek := nextChar := streamReader next]]
		ifFalse: [
			nextChar == $%
				ifTrue: [
					streamReader skipSeparators.
					referenceString := self nextLiteral.
					nextChar := self handleEntity: referenceString in: #entityValue.
					nextPeek := nextChar := streamReader next]
				ifFalse: [streamReader next]].
	nextPeek == delimiterChar]
		whileFalse: [
			nextChar ifNotNil: [entityValueStream nextPut: nextChar]].
	^ entityValueStream stringContents.! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 10/15/2010 15:32'!
nextExternalId
	| extDefType systemId dir |
	extDefType := self nextLiteral.
	extDefType = 'PUBLIC'
		ifTrue: [
			streamReader skipSeparators.
			self nextPubidLiteral.
			streamReader skipSeparators.
			streamReader peek == $>
				ifFalse: [
					systemId := self nextSystemLiteral]].

	extDefType = 'SYSTEM'
		ifTrue: [
			streamReader skipSeparators.
			systemId := self nextSystemLiteral].

	systemId
		ifNil: [^nil].

	"The rest of this method only applies if we're reading aFileStream"
	(streamReader stream isKindOf: FileStream)
		ifFalse: [^''].
	dir := streamReader stream directory.
	^(dir fileExists: systemId)
		ifTrue: [(dir readOnlyFileNamed: systemId) stringContents]
		ifFalse: ['']! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 09:11'!
nextIncludeSection: parseSection
	| section |
	"Read the file up to the next include section delimiter and parse it if parseSection is true"

	section := streamReader upToAll: ']]>'.
	parseSection
		ifTrue: [streamReader pushStream: (ReadStream on: section)]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/24/2010 16:24'!
nextLiteral
	| nextChar |

	((nextChar := streamReader peek) isLetter
		or: [nextChar == $_])
		ifFalse: [self errorExpected: 'name literal.'].

	^ streamWriter writeWith: [:writeStream |
		[LiteralChars includes: nextChar]
			whileTrue: [
				nextChar == $&
					ifTrue: [
						nextChar := streamReader next.
						writeStream nextPut: (streamReader peek == $#
							ifTrue: [self nextCharReference]
							ifFalse: [^ writeStream stringContents])]
					ifFalse: [writeStream nextPut: streamReader next].
				nextChar := streamReader peek].
		writeStream position > 0
			ifFalse: [self errorExpected: 'name literal'].
		writeStream stringContents]! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/11/2010 09:57'!
nextMarkupDeclaration
	| declType |
	declType := self nextLiteral.
	self isValidating
		ifFalse: [^ self skipMarkupDeclaration].
	declType = 'ENTITY'
		ifTrue: [self nextEntityDeclaration]
		ifFalse: [self skipMarkupDeclaration]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/19/2010 18:34'!
nextMarkupToken
	| nextChar |
	"Skip < "
	streamReader next.
	nextChar := streamReader peek.
	nextChar == $!! ifTrue: [
		"Skip !!"
		streamReader next.
		nextChar := streamReader peek.
		nextChar == $- ifTrue: [^ self nextComment].
		nextChar == $[ ifTrue: [^ self nextCDataOrConditional].
		^ self parsingMarkup
			ifTrue: [self nextMarkupDeclaration]
			ifFalse: [self nextDocType]].
	nextChar == $? ifTrue: [^ self nextPI].
	self nextTag.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/24/2010 16:25'!
nextName
	| nextChar |

	^ streamWriter writeWith: [:writeStream |
		[(nextChar := streamReader peek) isNil
			or: [NameDelimiters includes: nextChar]]
				whileFalse: [writeStream nextPut: streamReader next].
		writeStream position > 0
			ifFalse: [self errorExpected: 'name'].
		writeStream stringContents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/23/2010 18:04'!
nextPCData

	streamWriter writeWith: [:writeStream |
		self isValidating
			ifTrue: [self nextPCDataDelimitedBy: $< putOn: writeStream]
			ifFalse: [
				[streamReader peek == $<]
					whileFalse: [writeStream nextPut: streamReader next]].

		self handlePCData: writeStream stringContents]! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 16:45'!
nextPCDataDelimitedBy: aDelimiter putOn: aStream
	| nextChar referenceString entity entityValue |

	[(nextChar := streamReader peek) isNil or: [nextChar == aDelimiter]]
		whileFalse: [
			nextChar == $&
				ifTrue: [
					streamReader next.
					streamReader peek == $#
						ifTrue: [aStream nextPut: self nextCharReference]
						ifFalse: [
							referenceString := self nextLiteral.
							self expectNext: $;.
							entity := self entity: referenceString.
							entityValue := entity valueForContext: #content.
							(self class isCharEscape: entityValue)
								ifTrue: [aStream nextPut: entityValue first]
								ifFalse: [
									entityValue := entityValue asString.
									entityValue isEmpty
										ifFalse: [
											streamReader pushStream:
												(ReadStream on: entityValue asString)]]]]
				ifFalse: [aStream nextPut: streamReader next]].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/16/2010 09:11'!
nextPI
	| piTarget piData |
	"Skip ?"
	streamReader next.
	piTarget := self nextLiteral.
	piTarget asUppercase = 'XML'
		ifTrue: [^ self nextXMLDecl].
	streamReader skipSeparators.
	piData := streamReader upToAll: '?>'.
	self handlePI: piTarget data: piData.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:56'!
nextPubidLiteral
	^ self nextAttributeValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:56'!
nextSystemLiteral
	^ self nextAttributeValue! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/20/2010 08:03'!
nextTag
	| tagName attributes nextChar namespaces |
	(streamReader peek = $/)
		ifTrue: [^ self nextEndTag].
	tagName := self nextName.
	streamReader skipSeparators.
	attributes := OrderPreservingDictionary new: 10.
	self usesNamespaces
		ifTrue: [namespaces := OrderPreservingDictionary new].
	[(nextChar := streamReader peek) == $> or: [nextChar == $/]] whileFalse: [
		self checkAndExpandReference: #content.
		self nextAttributeInto: attributes namespaces: namespaces.
		streamReader skipSeparators.].
	self handleStartTag: tagName attributes: attributes namespaces: namespaces.
	streamReader next == $/
		ifTrue: [
			self handleEndTag: tagName.
			self expectNext: $>].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/19/2010 18:33'!
nextToken
	"return the next XMLnode, or nil if there are no more.
	Fixed to retain leading whitespace when PCDATA is detected."

	| whitespace |
	"branch, depending on what the first character is"
	whitespace := self nextWhitespace.
	streamReader atEnd
		ifTrue: [^ self handleEndDocument].

	self checkAndExpandReference: (self parsingMarkup ifTrue: [#dtd] ifFalse: [#content]).
	^ (streamReader peek = $<)
		ifTrue: [self nextMarkupToken]
		ifFalse: [
			whitespace ifNotEmpty: [streamReader pushBack: whitespace].
			self nextPCData].! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 3/11/2010 09:01'!
nextWhitespace
	| whitespace |

	(whitespace := streamReader nextWhitespace)
		ifNotEmpty: [self handleWhitespace: whitespace].
	^ whitespace.! !

!XMLTokenizer methodsFor: 'tokenizing' stamp: 'JAAyer 11/10/2010 05:46'!
nextXMLDecl
	| attributes namespaces |
	streamReader skipSeparators.
	attributes := Dictionary new.
	namespaces := Dictionary new.
	[streamReader peek == $?]
		whileFalse: [
			self nextAttributeInto: attributes namespaces: namespaces.
			streamReader skipSeparators].
	streamReader next.
	self expectNext: $>.
	(attributes includesKey: 'encoding')
		ifTrue: [streamReader streamEncoding: (attributes at: 'encoding')].
	self
		handleXMLVersion: (attributes at: 'version' ifAbsent: [''])
		encoding: (attributes at: 'encoding' ifAbsent: [''])
		standalone: (attributes at: 'standalone' ifAbsent: ['']).! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/14/2010 12:36'!
parameterEntities
	^ parameterEntities ifNil: [parameterEntities := Dictionary new]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/19/2010 14:17'!
parameterEntity: refName
	^ self parameterEntities
		at: refName
		ifAbsent: [self parseError: 'undefined parameter entity ', refName printString]! !

!XMLTokenizer methodsFor: 'entities' stamp: 'JAAyer 3/11/2010 09:55'!
parameterEntity: refName put: aReference
	"Only the first declaration of an entity is valid so if there is already
	one don't register the new value."
	self parameterEntities at: refName ifAbsentPut: [aReference]! !

!XMLTokenizer methodsFor: 'errors' stamp: 'JAAyer 11/22/2010 06:19'!
parseError: errorString
	XMLParsingException signal: errorString! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 3/11/2010 09:54'!
parsingMarkup
	^ parsingMarkup! !

!XMLTokenizer methodsFor: 'tokenizing dtd' stamp: 'JAAyer 3/11/2010 08:30'!
skipMarkupDeclaration
	streamReader skipUpTo: $>! !

!XMLTokenizer methodsFor: 'private' stamp: 'mir 11/13/2000 18:19'!
startParsingMarkup
	parsingMarkup := true! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 3/11/2010 08:04'!
stream
	^ streamReader stream! !

!XMLTokenizer methodsFor: 'private' stamp: 'JAAyer 3/11/2010 08:03'!
stream: aStream
	streamReader stream: aStream! !

!XMLTokenizer methodsFor: 'accessing' stamp: 'JAAyer 3/11/2010 09:03'!
streamReader
	^ streamReader! !

!XMLTokenizer methodsFor: 'testing' stamp: 'JAAyer 3/11/2010 09:54'!
usesNamespaces
	^ false! !

Object subclass: #XMLWellFormednessChecker
	instanceVariableNames: 'openElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'XML-Parser'!

!XMLWellFormednessChecker commentStamp: '<historical>' prior: 0!
The class ensures tags are nested properly and that all start tags have corresponding end tags.!

!XMLWellFormednessChecker methodsFor: 'printing' stamp: 'JAAyer 10/15/2010 15:13'!
enclosedElementsAsString: aTagName
	| enclosedTags |

	enclosedTags := String new writeStream.
	self
		printOn: enclosedTags
		startingAt: (self openElements lastIndexOf: aTagName) + 1.
	^ enclosedTags contents.! !

!XMLWellFormednessChecker methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:07'!
enclosingElement
	^ self openElements
		at: self totalOpen
		ifAbsent: ['']! !

!XMLWellFormednessChecker methodsFor: 'checking' stamp: 'JAAyer 10/15/2010 15:15'!
endedElement: anElementName
	(self enclosingElement = anElementName)
		ifFalse: [
			^ (self hasOpenElement: anElementName)
				ifTrue: [self errorImproperlyNestedElements: anElementName]
				ifFalse: [self errorUnmatchedEndTag: anElementName]].

	self openElements removeLast.! !

!XMLWellFormednessChecker methodsFor: 'checking' stamp: 'JAAyer 10/15/2010 15:14'!
ensureNoElementsRemainOpen
	self hasOpenElements
		ifTrue: [XMLWellFormednessException signal: 'Unterminated elements: ', self printString]! !

!XMLWellFormednessChecker methodsFor: 'errors' stamp: 'JAAyer 10/15/2010 15:13'!
errorImproperlyNestedElements: anElementName
	XMLWellFormednessException signal:
		('<{1}> element ended before its child {2} element(s)'
			format: (Array with: anElementName with: (self enclosedElementsAsString: anElementName)))! !

!XMLWellFormednessChecker methodsFor: 'errors' stamp: 'JAAyer 10/15/2010 15:15'!
errorUnmatchedEndTag: anElementName
	XMLWellFormednessException signal:
		('End </{1}> tag with no corresponding start <{1}> tag'
			format: (Array with: anElementName))! !

!XMLWellFormednessChecker methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 15:16'!
hasOpenElement: anElementName
	^ self openElements includes: anElementName! !

!XMLWellFormednessChecker methodsFor: 'testing' stamp: 'JAAyer 10/15/2010 15:15'!
hasOpenElements
	^ self openElements notEmpty! !

!XMLWellFormednessChecker methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:06'!
openElements
	^ openElements ifNil: [openElements := OrderedCollection new: 50]! !

!XMLWellFormednessChecker methodsFor: 'private' stamp: 'JAAyer 10/16/2010 06:50'!
printLimit
	^ 5! !

!XMLWellFormednessChecker methodsFor: 'printing' stamp: 'JAAyer 3/16/2010 16:07'!
printOn: aStream
	self printOn: aStream startingAt: 1! !

!XMLWellFormednessChecker methodsFor: 'printing' stamp: 'JAAyer 10/16/2010 06:50'!
printOn: aStream startingAt: aPosition
	| endPosition |

	endPosition :=  self totalOpen min: (aPosition + self printLimit - 1).
	(self openElements copyFrom: aPosition to: endPosition)
		do: [:each |
			aStream
				nextPut: $<;
				nextPutAll: each;
				nextPut: $>]
		separatedBy: [aStream nextPutAll: ', '].
	(endPosition < self totalOpen)
		ifTrue: [aStream nextPutAll: '...'].! !

!XMLWellFormednessChecker methodsFor: 'checking' stamp: 'JAAyer 10/15/2010 15:10'!
startedElement: aTagName
	^ self openElements addLast: aTagName! !

!XMLWellFormednessChecker methodsFor: 'accessing' stamp: 'JAAyer 10/15/2010 15:06'!
totalOpen
	^ self openElements size
	! !
XMLNestedStreamReader initialize!
XMLTokenizer initialize!
DTDEntityDeclaration initialize!
DTDParameterEntityDeclaration initialize!
DTDExternalEntityDeclaration initialize!
